"""
This type stub file was generated by pyright.
"""

from typing import Optional
from typing import Sequence
from typing import Union

import abc
from dataclasses import dataclass
from enum import Enum
from os import PathLike

class ToastButtonColour(Enum):
    """
    Possible colours for toast buttons
    """

    Default = ...
    Green = ...
    Red = ...

class ToastDuration(Enum):
    """
    Possible values for duration to display toast for
    """

    Default: str = ...
    Short: str = ...
    Long: str = ...

class ToastImagePosition(Enum):
    """
    Allowed positions for an image to be placed on a toast notification
    """

    Inline = ...
    Hero = ...
    AppLogo = ...

class ToastScenario(Enum):
    """
    Possible scenarios for the toast
    """

    Default: str = ...
    Alarm: str = ...
    Reminder: str = ...
    IncomingCall: str = ...
    Important: str = ...

class ToastSystemButtonAction(Enum):
    Snooze = ...
    Dismiss = ...

@dataclass(init=False)
class ToastImage:
    """
    Image that can be displayed in various toast elements
    """

    path: str
    def __init__(self, imagePath: Union[str, PathLike]) -> None:
        """
        Initialise an :class:`ToastImage` class to use in certain classes.
        Online images are supported only in packaged apps that have the internet capability in their manifest.
        Unpackaged apps don't support http images; you must download the image to your local app data,
        and reference it locally.

        :param imagePath: The path to an image file
        :type imagePath: Union[str, PathLike]
        :raises: InvalidImageException: If the path to an online image is supplied
        """
        ...

@dataclass
class ToastDisplayImage:
    """
    Define an image that will be displayed as the icon of the toast
    """

    image: ToastImage
    altText: Optional[str] = ...
    position: ToastImagePosition = ...
    circleCrop: bool = ...
    @classmethod
    def fromPath(
        cls,
        imagePath: Union[str, PathLike],
        altText: Optional[str] = ...,
        position: ToastImagePosition = ...,
        circleCrop: bool = ...,
    ) -> ToastDisplayImage:
        """
        Create a :class:`ToastDisplayImage` object from path without having to create :class:`ToastImage`
        """
        ...

@dataclass
class ToastProgressBar:
    """
    Progress bar to be included in a toast
    """

    status: str
    caption: Optional[str] = ...
    progress: Optional[float] = ...
    progress_override: Optional[str] = ...

@dataclass
class _ToastInput(abc.ABC):
    """
    Base input dataclass to be used in toasts
    """

    input_id: str
    caption: str = ...

@dataclass(init=False)
class ToastInputTextBox(_ToastInput):
    """
    A text box that can be added in toasts for the user to enter their input
    """

    placeholder: str = ...
    def __init__(
        self, input_id: str, caption: str = ..., placeholder: str = ...
    ) -> None: ...

@dataclass
class ToastSelection:
    """
    An item that the user can select from the drop down list
    """

    selection_id: str
    content: str
    ...

@dataclass(init=False)
class ToastInputSelectionBox(_ToastInput):
    """
    A selection box control, which lets users pick from a dropdown list of options
    """

    selections: Sequence[ToastSelection] = ...
    default_selection: Optional[ToastSelection] = ...
    def __init__(
        self,
        input_id: str,
        caption: str = ...,
        selections: Sequence[ToastSelection] = ...,
        default_selection: Optional[ToastSelection] = ...,
    ) -> None: ...

@dataclass
class ToastButton:
    """
    A button that the user can click on a toast notification
    """

    content: str = ...
    arguments: str = ...
    image: Optional[ToastImage] = ...
    relatedInput: Optional[Union[ToastInputTextBox, ToastInputSelectionBox]] = ...
    inContextMenu: bool = ...
    tooltip: Optional[str] = ...
    launch: Optional[str] = ...
    colour: ToastButtonColour = ...

@dataclass
class ToastSystemButton:
    """
    Button used to perform a system action, snooze or dismiss
    """

    action: ToastSystemButtonAction
    content: str = ...
    relatedInput: Optional[ToastInputSelectionBox] = ...
    image: Optional[ToastImage] = ...
    tooltip: Optional[str] = ...
    colour: ToastButtonColour = ...



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import unittest
from unittest.mock import MagicMock

from nbxmpp.protocol import JID

from gajim.common import app  # type: ignore # noqa: F401
from gajim.common import events
from gajim.common.modules.contacts import GroupchatContact
from gajim.common.storage.events.storage import EventStorage
from gajim.common.util.datetime import utc_now


class EventsTest(unittest.TestCase):
    def setUp(self) -> None:
        self._event_storage = EventStorage()
        self._event_storage.init()

        self._account = "testacc1"

        self._group_chat_contact = MagicMock(spec_set=GroupchatContact)
        self._group_chat_contact.account = self._account
        self._group_chat_contact.jid = JID.from_string("groupchat@example.org")

    def tearDown(self) -> None:
        self._event_storage.shutdown()

    def test_insert_muc_room_destroyed(self) -> None:
        event_data = events.MUCRoomDestroyed(
            timestamp=utc_now(),
            reason="Reason",
            alternate=JID.from_string("some-alternate@example.org"),
        )

        self._event_storage.store(self._group_chat_contact, event_data)

        events_list, _complete = self._event_storage.load(
            self._group_chat_contact, "before", utc_now().timestamp(), 1
        )
        first_event = events_list[0]
        assert isinstance(first_event, events.MUCRoomDestroyed)
        self.assertEqual(
            first_event.alternate, JID.from_string("some-alternate@example.org")
        )


if __name__ == "__main__":
    unittest.main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unicodedata

from gi.repository import Gtk
from gi.repository import Pango

from gajim.common.util.text import process_non_spacing_marks

from gajim.gtk.window import GajimAppWindow

from . import util

FATAL_STRING = "aر ॣॣb"


class TestPangoWordWrap(GajimAppWindow):
    def __init__(self) -> None:
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=600,
            default_height=600,
            add_window_padding=True,
            header_bar=True,
        )

        box = Gtk.Box(
            halign=Gtk.Align.CENTER,
            valign=Gtk.Align.CENTER,
            orientation=Gtk.Orientation.VERTICAL,
            hexpand=True,
            spacing=12,
        )
        self.set_child(box)

        label1 = Gtk.Label(
            label=FATAL_STRING, wrap=True, wrap_mode=get_pango_wrap_mode(FATAL_STRING)
        )
        box.append(label1)

        label2 = Gtk.Label(
            label=process_non_spacing_marks(FATAL_STRING),
            wrap=True,
            wrap_mode=Pango.WrapMode.WORD_CHAR,
        )
        box.append(label2)

    def _cleanup(self) -> None:
        pass


def get_pango_wrap_mode(string: str) -> Pango.WrapMode:
    for char in string:
        if unicodedata.category(char) == "Mn":
            return Pango.WrapMode.WORD
    return Pango.WrapMode.WORD_CHAR


util.init_settings()

window = TestPangoWordWrap()
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import Gtk
from nbxmpp import JID
from nbxmpp.const import InviteType
from nbxmpp.modules.discovery import parse_disco_info
from nbxmpp.protocol import Iq

from gajim.common import app
from gajim.common.events import MucInvitation
from test.gtk.application import GajimTestApplication

from gajim.gtk.groupchat_invitation import GroupChatInvitation
from gajim.gtk.window import GajimAppWindow

MUC_JID = JID.from_string("group@conference.example.org")

stanza = Iq(
    node=f"""
<iq xmlns="jabber:client" xml:lang="de-DE" to="me@example.org" from="{MUC_JID}" type="result" id="67284933-e526-41f3-8309-9d9475cf9c74">
    <query
        xmlns="http://jabber.org/protocol/disco#info">
        <identity name="ipsum dolor sit amet, consetetur sadipscing elitr sed diam nonumy eirmod tempor invidunt" type="text" category="conference" />
        <feature var="vcard-temp" />
        <feature var="http://jabber.org/protocol/muc" />
        <feature var="http://jabber.org/protocol/disco#info" />
        <feature var="http://jabber.org/protocol/disco#items" />
        <feature var="muc_temporary" />
        <feature var="muc_moderated" />
        <feature var="muc_open" />
        <feature var="muc_hidden" />
        <feature var="muc_nonanonymous" />
        <feature var="muc_passwordprotected" />
        <feature var="urn:xmpp:mam:2" />
        <feature var="muc_public" />
        <feature var="muc_persistent" />
        <feature var="muc_membersonly" />
        <feature var="muc_semianonymous" />
        <feature var="muc_unmoderated" />
        <feature var="muc_unsecured" />
        <x type="result"
            xmlns="jabber:x:data">
            <field var="FORM_TYPE" type="hidden">
                <value>http://jabber.org/protocol/muc#roominfo</value>
            </field>
            <field var="muc#roominfo_occupants" type="text-single" label="Number of occupants">
                <value>1</value>
            </field>
            <field var="muc#roomconfig_roomname" type="text-single" label="Natural-Language Room Name">
                <value>ipsum dolor sit amet, consetetur sadipscing elitr sed diam nonumy eirmod tempor invidunt</value>
            </field>
            <field var="muc#roominfo_description" type="text-single" label="Raum Beschreibung">
                <value>Lorem ipsum dolor sit amet, consetetur sadipscing elitr sed diam nonumy eirmod tempor invidunt ut labore et dolore magna</value>
            </field>
            <field var="muc#roominfo_contactjid" type="jid-multi" label="Contact Addresses (normally, room owner or owners)">
                <value>userA@user.us</value>
                <value>userB@user.us</value>
            </field>
            <field var="muc#roominfo_changesubject" type="boolean" label="Occupants May Change the Subject">
                <value>1</value>
            </field>
            <field var="muc#roomconfig_allowinvites" type="boolean" label="Occupants are allowed to invite others">
                <value>1</value>
            </field>
            <field var="muc#roomconfig_allowpm" type="list-single" label="Roles that May Send Private Messages">
                <value>anyone</value>
                <option label="Anyone">
                    <value>anyone</value>
                </option>
                <option label="Anyone with Voice">
                    <value>participants</value>
                </option>
                <option label="Moderators Only">
                    <value>moderators</value>
                </option>
                <option label="Nobody">
                    <value>none</value>
                </option>
            </field>
            <field var="muc#roominfo_lang" type="text-single" label="Natural Language for Room Discussions">
                <value>de</value>
            </field>
            <field type="text-single" var="muc#roominfo_logs">
                <value>https://logs.xmpp.org/xsf/</value>
            </field>
        </x>
    </query>
</iq>"""  # type: ignore  # noqa: E501
)


subject = (
    "Lorem ipsum dolor sit amet, consetetur sadipscing elitr sed "
    "diam nonumy eirmod tempor invidunt ut labore et dolore magna"
)

disco_info = parse_disco_info(stanza)

ACCOUNT = "testacc0"


class TestGroupchatInvitation(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=700,
            default_height=700,
            add_window_padding=True,
            header_bar=True,
        )

        main_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=18)
        main_box.set_valign(Gtk.Align.FILL)

        event = MucInvitation(
            account=ACCOUNT,
            info=disco_info,
            muc=MUC_JID,
            from_=JID.from_string("contact@example.org"),
            reason="Reason",
            password=None,
            type=InviteType.MEDIATED,
            continued=False,
            thread=None,
        )
        invitation_box = GroupChatInvitation(ACCOUNT, event)
        invitation_box.set_vexpand(True)

        main_box.append(invitation_box)

        self.set_child(main_box)

    def _cleanup(self) -> None:
        pass


def init(app: GajimTestApplication) -> None:
    app.add_account("testacc0")
    app.add_account("testacc1")
    window = TestGroupchatInvitation()
    window.present()


if __name__ == "__main__":
    # Protect the entry point of the application because we use
    # the multiprocessing module with "spawn"
    app = GajimTestApplication()
    app.connect("activate", init)
    app.run(None)



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later
import typing

from pathlib import Path
from unittest.mock import MagicMock

import gi

try:
    gi.require_version("Gst", "1.0")
    from gi.repository import Gst
except Exception:
    if typing.TYPE_CHECKING:
        from gi.repository import Gst

from gi.repository import Gst
from gi.repository import Gtk

from gajim.common import app

from gajim.gtk.audio_player import AudioPlayer
from gajim.gtk.filechoosers import FileChooserButton
from gajim.gtk.filechoosers import Filter
from gajim.gtk.preview.audio import AudioPreviewWidget
from gajim.gtk.window import GajimAppWindow

from . import util

DEFAULT_AUDIO_FILE_PATH = util.get_gajim_dir() / "data/sounds/attention.wav"


class TestAudioWidget(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=600,
            default_height=600,
            add_window_padding=True,
            header_bar=True,
        )

        self._box = Gtk.Box(
            halign=Gtk.Align.CENTER,
            valign=Gtk.Align.CENTER,
            hexpand=True,
            orientation=Gtk.Orientation.VERTICAL,
            spacing=12,
        )
        self.set_child(self._box)

        self._audio_player = AudioPlayer()

        file_chooser_button = FileChooserButton(
            label="Select Audio File",
            filters=[
                Filter(name="All files", patterns=["*"]),
                Filter(name="WAV Sounds", suffixes=["wav"]),
                Filter(name="MP3 Sounds", suffixes=["mp3"], default=True),
            ],
            path=DEFAULT_AUDIO_FILE_PATH,
        )
        file_chooser_button.connect("path-picked", self._on_path_picked)
        self._box.append(file_chooser_button)

        self._audio_widget = AudioPreviewWidget(
            self._audio_player,
            DEFAULT_AUDIO_FILE_PATH.name,
            DEFAULT_AUDIO_FILE_PATH.stat().st_size,
            DEFAULT_AUDIO_FILE_PATH,
        )
        self._box.append(self._audio_widget)

    def _on_path_picked(self, _button: FileChooserButton, paths: list[Path]) -> None:
        self._box.remove(self._audio_widget)
        del self._audio_widget

        self._audio_widget = AudioPreviewWidget(
            self._audio_player, paths[0].as_posix(), paths[0].stat().st_size, paths[0]
        )
        self._box.append(self._audio_widget)

    def _cleanup(self) -> None:
        pass


util.init_settings()

Gst.init()

app.is_installed = MagicMock(return_value=True)

app.init_process_pool()

window = TestAudioWidget()
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from unittest.mock import MagicMock

from gi.repository import Gtk
from gi.repository import Pango

from gajim.common import app
from gajim.common.const import StyleAttr

from gajim.gtk.themes import Themes

from . import util

ACCOUNT = "test"

util.init_settings()
app.settings.get_workspace_count = MagicMock(return_value=2)


def _get_value(
    selector: str, attr: str | StyleAttr, pre: bool = False
) -> str | Pango.FontDescription | None:
    if attr == StyleAttr.FONT:
        return Pango.FontDescription.from_string("Cantarell Italic Light 15")
    else:
        return "rgb(100, 10, 50)"


app.css_config = MagicMock()
app.css_config.get_value = _get_value

window = Themes(Gtk.Window())
window._update_preferences_window = MagicMock()  # type: ignore
window.show()

util.run_app()



from __future__ import annotations

import typing

import winrt.system

TResult = typing.TypeVar("TResult")
TSender = typing.TypeVar("TSender")
TypedEventHandler = typing.Callable[
    [typing.Optional[TSender], typing.Optional[TResult]], None
]

@typing.final
class EventRegistrationToken:
    value: winrt.system.Int64



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import datetime as dt
import unittest
from datetime import datetime
from datetime import timedelta
from datetime import UTC

import sqlalchemy.exc
from nbxmpp.protocol import JID
from sqlalchemy import select
from sqlalchemy.orm import selectinload

import gajim.common.storage.archive.models as mod
from gajim.common import app
from gajim.common.helpers import get_uuid
from gajim.common.settings import Settings
from gajim.common.storage.archive.const import ChatDirection
from gajim.common.storage.archive.const import MessageState
from gajim.common.storage.archive.const import MessageType
from gajim.common.storage.archive.storage import MessageArchiveStorage
from gajim.common.util.datetime import utc_now


class MethodsTest(unittest.TestCase):
    def setUp(self) -> None:
        self._archive = MessageArchiveStorage(in_memory=True)
        self._archive.init()

        self._account = "testacc1"
        self._occupant_id = "occupantid1"
        self._init_settings()

    def tearDown(self) -> None:
        self._archive.shutdown()

    def _init_settings(self) -> None:
        app.settings = Settings(in_memory=True)
        app.settings.init()
        app.settings.add_account("testacc1")
        app.settings.set_account_setting("testacc1", "address", "user@domain.org")
        app.settings.add_account("testacc2")
        app.settings.set_account_setting("testacc2", "address", "user2@domain.org")

    def _insert_messages(
        self,
        account: str = "testacc1",
        remote_jid: JID | None = None,
        resource: str | None = None,
        message: str = "test",
        message_id: str | None = None,
        type: int = MessageType.CHAT,  # noqa: A002
        timestamp: datetime | None = None,
        count: int = 10,
    ) -> None:
        for i in range(count):
            remote_jid_ = remote_jid
            if remote_jid_ is None:
                remote_jid_ = JID.from_string(f"remote{i}@jid.org")

            timestamp_ = timestamp
            if timestamp_ is None:
                timestamp_ = datetime.now(dt.UTC)

            message_id_ = message_id
            if message_id_ is None:
                message_id_ = f"messageid{i}"

            resource_ = resource
            if resource_ is None:
                resource_ = f"res{i}"

            m = mod.Message(
                account_=account,
                remote_jid_=remote_jid_,
                resource=resource_,
                type=type,
                direction=ChatDirection.INCOMING,
                timestamp=timestamp_,
                state=MessageState.ACKNOWLEDGED,
                id=message_id_,
                text=message,
            )
            self._archive.insert_object(m)

    def test_rollback(self) -> None:
        now = utc_now()
        uuid = get_uuid()
        m1 = mod.Message(
            account_="testacc1",
            remote_jid_=JID.from_string("remote1@jid.org"),
            resource=None,
            type=MessageType.CHAT,
            direction=ChatDirection.INCOMING,
            timestamp=now,
            state=MessageState.ACKNOWLEDGED,
            id="1",
            stanza_id=uuid,
        )
        pk = self._archive.insert_object(m1)

        m2 = mod.Message(
            account_="testacc1",
            remote_jid_=JID.from_string("remote1@jid.org"),
            resource=None,
            type=MessageType.CHAT,
            direction=ChatDirection.INCOMING,
            timestamp=now,
            state=MessageState.ACKNOWLEDGED,
            id="1",
            stanza_id=uuid,
            encryption_=mod.Encryption(protocol="OMEMO", key="123", trust=1),
        )
        m2.pk = pk

        with self.assertRaises(sqlalchemy.exc.IntegrityError):
            self._archive.insert_object(m2, ignore_on_conflict=False)

        with self._archive.get_session() as s:
            result = s.scalar(select(mod.Encryption))
            self.assertIsNone(result)

    def test_get_conversation_jids(self) -> None:
        self._insert_messages("testacc1", count=10)
        self._insert_messages("testacc2", count=12)

        rows = self._archive.get_conversation_jids("testacc1")
        self.assertEqual(len(rows), 10)
        rows = self._archive.get_conversation_jids("testacc2")
        self.assertEqual(len(rows), 12)

    def test_get_conversation_before_after(self) -> None:
        remote_jid = JID.from_string("remote1@jid.org")
        self._insert_messages("testacc1", remote_jid=remote_jid, count=4)

        messages, complete = self._archive.get_conversation_before_after(
            "testacc1",
            remote_jid,
            datetime.now(dt.UTC),
            2,
            direction="before",
            order="desc",
        )

        messages = list(messages)
        self.assertEqual(len(messages), 2)
        self.assertEqual(messages[0].id, "messageid3")
        self.assertEqual(messages[1].id, "messageid2")
        self.assertFalse(complete)

        messages, complete = self._archive.get_conversation_before_after(
            "testacc1",
            remote_jid,
            datetime(year=1970, month=1, day=1, tzinfo=dt.UTC),
            2,
            direction="after",
            order="desc",
        )

        messages = list(messages)
        self.assertEqual(len(messages), 2)
        self.assertEqual(messages[0].id, "messageid1")
        self.assertEqual(messages[1].id, "messageid0")
        self.assertFalse(complete)

        messages, complete = self._archive.get_conversation_before_after(
            "testacc1",
            remote_jid,
            datetime.now(dt.UTC),
            2,
            direction="before",
            order="asc",
        )

        messages = list(messages)
        self.assertEqual(len(messages), 2)
        self.assertEqual(messages[0].id, "messageid2")
        self.assertEqual(messages[1].id, "messageid3")
        self.assertFalse(complete)

        messages, complete = self._archive.get_conversation_before_after(
            "testacc1",
            remote_jid,
            datetime(year=1970, month=1, day=1, tzinfo=dt.UTC),
            2,
            direction="after",
            order="asc",
        )

        messages = list(messages)
        self.assertEqual(len(messages), 2)
        self.assertEqual(messages[0].id, "messageid0")
        self.assertEqual(messages[1].id, "messageid1")
        self.assertFalse(complete)

        # Test complete flag

        # Same amount returned as requested
        messages, complete = self._archive.get_conversation_before_after(
            "testacc1",
            remote_jid,
            datetime(year=1970, month=1, day=1, tzinfo=dt.UTC),
            4,
            direction="after",
            order="asc",
        )

        messages = list(messages)
        self.assertEqual(len(messages), 4)
        self.assertEqual(messages[0].id, "messageid0")
        self.assertEqual(messages[1].id, "messageid1")
        self.assertFalse(complete)

        # Less returned than requested
        messages, complete = self._archive.get_conversation_before_after(
            "testacc1",
            remote_jid,
            datetime(year=1970, month=1, day=1, tzinfo=dt.UTC),
            5,
            direction="after",
            order="asc",
        )

        messages = list(messages)
        self.assertEqual(len(messages), 4)
        self.assertEqual(messages[0].id, "messageid0")
        self.assertEqual(messages[1].id, "messageid1")
        self.assertTrue(complete)

    def test_get_last_conversation_row(self) -> None:
        remote_jid = JID.from_string("remote1@jid.org")
        self._insert_messages(
            "testacc1",
            remote_jid=remote_jid,
            timestamp=datetime.now(dt.UTC),
            count=10,
        )

        message = self._archive.get_last_conversation_row(
            "testacc1", remote_jid, incl_related_data=True
        )
        assert message is not None

        self.assertEqual(message.id, "messageid9")

    def test_search_archive(self) -> None:
        # TODO
        return
        # remote_jid = JID.from_string(f'remote1@jid.org')
        # self._insert_messages(
        #     'testacc1', remote_jid=remote_jid, message='test', count=100)

        # iterator = self._archive.search_archive('testacc1', remote_jid, 'test')
        # messages = list(iterator)

    def test_get_days_containing_messages(self) -> None:
        localtime = datetime(2023, 12, 31, 23, 59, 59, tzinfo=dt.UTC).astimezone()
        offset = localtime.utcoffset()
        assert offset is not None
        offset_s = offset.total_seconds()

        remote_jid = JID.from_string("remote1@jid.org")

        timestamp = datetime(2023, 12, 30, 23, 59, 59, 0, tzinfo=dt.UTC)
        for day in range(1, 5):
            timestamp += timedelta(days=1)
            self._insert_messages(
                "testacc1",
                remote_jid=remote_jid,
                timestamp=timestamp,
                message_id=f"messageid{day}",
                count=1,
            )

        days = self._archive.get_days_containing_messages(
            "testacc1", remote_jid, 2024, 1
        )

        self.assertEqual(
            days,
            [1, 2, 3] if offset_s <= 0 else [1, 2, 3, 4],
            msg=f"Localtime: {localtime}, offset: {offset_s}",
        )

    def test_get_last_message_ts(self) -> None:
        remote_jid = JID.from_string("remote1@jid.org")

        timestamp = datetime(2024, 1, 1, tzinfo=dt.UTC)
        for day in range(2, 5):
            timestamp += timedelta(days=1)
            self._insert_messages(
                "testacc1",
                remote_jid=remote_jid,
                timestamp=timestamp,
                message_id=f"messageid{day}",
                count=1,
            )

        timestamp = self._archive.get_last_message_ts("testacc1", remote_jid)

        self.assertEqual(timestamp, datetime(2024, 1, 4, tzinfo=dt.UTC))

    def test_get_first_message_ts(self) -> None:
        remote_jid = JID.from_string("remote1@jid.org")

        timestamp = datetime(2024, 1, 1, tzinfo=dt.UTC)
        for day in range(2, 5):
            timestamp += timedelta(days=1)
            self._insert_messages(
                "testacc1",
                remote_jid=remote_jid,
                timestamp=timestamp,
                message_id=f"messageid{day}",
                count=1,
            )

        timestamp = self._archive.get_first_message_ts("testacc1", remote_jid)

        self.assertEqual(timestamp, datetime(2024, 1, 2, tzinfo=dt.UTC))

    def test_get_first_message_meta_for_date(self) -> None:
        localtime = datetime(2023, 12, 31, 23, 59, 59, tzinfo=dt.UTC).astimezone()
        offset = localtime.utcoffset()
        assert offset is not None
        offset_s = offset.total_seconds()

        remote_jid = JID.from_string("remote1@jid.org")

        timestamp = datetime(2023, 12, 30, 23, 59, 59, 0, tzinfo=dt.UTC)
        for day in range(1, 5):
            timestamp += timedelta(days=1)
            self._insert_messages(
                "testacc1",
                remote_jid=remote_jid,
                timestamp=timestamp,
                message_id=f"messageid{day}",
                count=1,
            )

        metadata = self._archive.get_first_message_meta_for_date(
            "testacc1", remote_jid, dt.date(2024, 1, 1)
        )
        assert metadata is not None
        pk, timestamp = metadata

        if offset_s > 0:
            self.assertEqual(pk, 1)
            self.assertEqual(
                timestamp, datetime(2023, 12, 31, 23, 59, 59, 0, tzinfo=dt.UTC)
            )
        else:
            self.assertEqual(pk, 2)
            self.assertEqual(
                timestamp, datetime(2024, 1, 1, 23, 59, 59, 0, tzinfo=dt.UTC)
            )

        metadata = self._archive.get_first_message_meta_for_date(
            "testacc1", remote_jid, dt.date(2024, 2, 1)
        )
        self.assertIsNone(metadata)

    def test_get_recent_muc_nicks(self) -> None:
        remote_jid = JID.from_string("remote1@jid.org")

        timestamp = datetime.now(dt.UTC) - timedelta(days=91)

        self._insert_messages(
            "testacc1",
            remote_jid=remote_jid,
            timestamp=timestamp,
            resource="res101",
            message_id="messageid101",
            count=1,
        )
        self._insert_messages(
            "testacc1", remote_jid=remote_jid, type=MessageType.GROUPCHAT, count=10
        )
        nicknames = self._archive.get_recent_muc_nicks("testacc1", remote_jid)
        self.assertEqual(len(nicknames), 10)
        self.assertNotIn("res101", nicknames)

    def test_get_mam_archive_state(self) -> None:
        remote_jid = JID.from_string("remote1@jid.org")
        state = mod.MAMArchiveState(
            account_="testacc1",
            remote_jid_=remote_jid,
            from_stanza_id="stanzaid1",
            from_stanza_ts=datetime.now(dt.UTC),
            to_stanza_id="stanzaid2",
            to_stanza_ts=datetime.now(dt.UTC),
        )
        self._archive.upsert_row(state)

        state = self._archive.get_mam_archive_state("testacc1", remote_jid)
        assert state is not None
        self.assertEqual(state.from_stanza_id, "stanzaid1")
        self.assertEqual(state.to_stanza_id, "stanzaid2")

    def test_reset_mam_archive_state(self) -> None:
        remote_jid = JID.from_string("remote1@jid.org")
        state = mod.MAMArchiveState(
            account_="testacc1",
            remote_jid_=remote_jid,
        )
        self._archive.upsert_row(state)

        state = self._archive.get_mam_archive_state("testacc1", remote_jid)
        assert state is not None
        self._archive.reset_mam_archive_state("testacc1", remote_jid)
        state = self._archive.get_mam_archive_state("testacc1", remote_jid)
        self.assertIsNone(state)

    def test_remove_history_for_jid(self) -> None:
        remote_jid = JID.from_string("remote1@jid.org")
        self._insert_messages("testacc1", remote_jid=remote_jid, count=10)

        # Moderation message should no match any message to
        # test if orphan data are removed
        m = mod.Moderation(
            account_="testacc1",
            remote_jid_=remote_jid,
            occupant_=None,
            stanza_id="DoNotMatchID",
            by=None,
            reason=None,
            timestamp=utc_now(),
        )
        self._archive.insert_object(m)

        error = mod.MessageError(
            account_="testacc1",
            remote_jid_=remote_jid,
            message_id="messageid1",
            by=None,
            type="modify",
            text="text",
            condition="somecond",
            condition_text="somecondtext",
            timestamp=utc_now(),
        )

        self._archive.insert_object(error)

        seclabel = mod.SecurityLabel(
            account_="testacc1",
            remote_jid_=remote_jid,
            label_hash="hash1",
            displaymarking="dm",
            bgcolor="red",
            fgcolor="blue",
            updated_at=utc_now(),
        )

        reaction = mod.Reaction(
            account_=self._account,
            remote_jid_=remote_jid,
            occupant_=None,
            id="123",
            direction=ChatDirection.INCOMING,
            emojis="😁️;😘️;😇️",
            timestamp=utc_now(),
        )

        self._archive.upsert_row2(reaction, return_full=False)

        marker = mod.DisplayedMarker(
            account_=self._account,
            remote_jid_=remote_jid,
            occupant_=None,
            id="123",
            timestamp=utc_now(),
        )

        self._archive.insert_object(marker)

        receipt = mod.Receipt(
            account_=self._account,
            remote_jid_=remote_jid,
            id="123",
            timestamp=utc_now(),
        )

        self._archive.insert_object(receipt)

        retraction_data = mod.Retraction(
            account_=self._account,
            remote_jid_=remote_jid,
            occupant_=None,
            id="123",
            direction=ChatDirection.OUTGOING,
            timestamp=utc_now(),
        )

        self._archive.insert_row(retraction_data)

        m = mod.Message(
            account_="testacc1",
            remote_jid_=remote_jid,
            resource="test",
            type=MessageType.CHAT,
            direction=ChatDirection.INCOMING,
            timestamp=utc_now(),
            state=MessageState.ACKNOWLEDGED,
            id="123",
            stanza_id="stanzaid123",
            text="testmessage",
            security_label_=seclabel,
            thread_id_="testthreadid",
            encryption_=mod.Encryption(protocol="OMEMO", key="somekey", trust=1),
            call=mod.Call(sid="somesid", state=1),
            oob=[mod.OOB(url="someurl", description="desc")],
            og=[mod.OpenGraph(about="about", title="title")],
            reply=mod.Reply(id="132", to=None),
        )
        self._archive.insert_object(m)

        jid_related_tables = [
            mod.OOB,
            mod.OpenGraph,
            mod.Reply,
            mod.Call,
            mod.MessageError,
            mod.Moderation,
            mod.Retraction,
            mod.Receipt,
            mod.DisplayedMarker,
            mod.Reaction,
            mod.Message,
            mod.SecurityLabel,
            mod.Thread,
        ]

        with self._archive.get_session() as s:
            for table in jid_related_tables:
                result = s.scalar(select(table))
                self.assertIsNotNone(result)

            result = s.scalar(select(mod.Encryption))
            self.assertIsNotNone(result)

        self._archive.remove_history_for_jid("testacc1", remote_jid)

        with self._archive.get_session() as s:
            for table in jid_related_tables:
                result = s.scalar(select(table))
                self.assertIsNone(result)

            result = s.scalar(select(mod.Encryption))
            self.assertIsNotNone(result)

    def test_delete_message(self) -> None:
        remote_jid = JID.from_string("remote1@jid.org")

        m = mod.Moderation(
            account_="testacc1",
            remote_jid_=remote_jid,
            occupant_=None,
            stanza_id="stanzaid123",
            by=None,
            reason=None,
            timestamp=utc_now(),
        )
        self._archive.insert_object(m)

        error = mod.MessageError(
            account_="testacc1",
            remote_jid_=remote_jid,
            message_id="123",
            by=None,
            type="modify",
            text="text",
            condition="somecond",
            condition_text="somecondtext",
            timestamp=utc_now(),
        )

        self._archive.insert_object(error)

        reaction = mod.Reaction(
            account_=self._account,
            remote_jid_=remote_jid,
            occupant_=None,
            id="stanzaid123",
            direction=ChatDirection.INCOMING,
            emojis="😁️;😘️;😇️",
            timestamp=utc_now(),
        )

        self._archive.upsert_row2(reaction, return_full=False)

        marker = mod.DisplayedMarker(
            account_=self._account,
            remote_jid_=remote_jid,
            occupant_=None,
            id="stanzaid123",
            timestamp=utc_now(),
        )

        self._archive.insert_object(marker)

        receipt = mod.Receipt(
            account_=self._account,
            remote_jid_=remote_jid,
            id="123",
            timestamp=utc_now(),
        )

        self._archive.insert_object(receipt)

        retraction_data = mod.Retraction(
            account_=self._account,
            remote_jid_=remote_jid,
            occupant_=None,
            id="stanzaid123",
            direction=ChatDirection.INCOMING,
            timestamp=utc_now(),
        )

        self._archive.insert_row(retraction_data)

        m = mod.Message(
            account_="testacc1",
            remote_jid_=remote_jid,
            resource="test",
            type=MessageType.GROUPCHAT,
            direction=ChatDirection.INCOMING,
            timestamp=utc_now(),
            state=MessageState.ACKNOWLEDGED,
            id="123",
            stanza_id="stanzaid123",
            text="testmessage",
            security_label_=mod.SecurityLabel(
                account_="testacc1",
                remote_jid_=remote_jid,
                label_hash="hash1",
                displaymarking="dm",
                bgcolor="red",
                fgcolor="blue",
                updated_at=utc_now(),
            ),
            thread_id_="testthreadid",
            encryption_=mod.Encryption(protocol="OMEMO", key="somekey", trust=1),
            call=mod.Call(sid="somesid", state=1),
            oob=[mod.OOB(url="someurl", description="desc")],
            og=[mod.OpenGraph(about="about", title="title")],
            reply=mod.Reply(id="132", to=None),
        )
        m_pk = self._archive.insert_object(m)

        m = mod.Message(
            account_="testacc1",
            remote_jid_=remote_jid,
            resource="test",
            type=MessageType.GROUPCHAT,
            direction=ChatDirection.INCOMING,
            timestamp=utc_now(),
            state=MessageState.ACKNOWLEDGED,
            id="124",
            stanza_id="stanzaid124",
            correction_id="123",
            text="corrected message",
            thread_id_="testthreadid",
            call=mod.Call(sid="somesid", state=1),
            oob=[mod.OOB(url="someurl", description="desc")],
            og=[mod.OpenGraph(about="about", title="title")],
            reply=mod.Reply(id="132", to=None),
        )

        self._archive.insert_object(m)

        reaction = mod.Reaction(
            account_=self._account,
            remote_jid_=remote_jid,
            occupant_=None,
            id="stanzaid124",
            direction=ChatDirection.INCOMING,
            emojis="😁️",
            timestamp=utc_now(),
        )

        self._archive.upsert_row2(reaction, return_full=False)

        jid_related_tables = [
            mod.OOB,
            mod.OpenGraph,
            mod.Reply,
            mod.Call,
            mod.MessageError,
            mod.Retraction,
            mod.Receipt,
            mod.DisplayedMarker,
            mod.Reaction,
            mod.Moderation,
            mod.Message,
        ]

        many_to_one_tables = [
            mod.Encryption,
            mod.SecurityLabel,
            mod.Thread,
        ]

        with self._archive.get_session() as s:
            for table in jid_related_tables:
                result = s.scalar(select(table))
                self.assertIsNotNone(result)

            for table in many_to_one_tables:
                result = s.scalar(select(table))
                self.assertIsNotNone(result)

        self._archive.delete_message(m_pk)

        with self._archive.get_session() as s:
            for table in jid_related_tables:
                result = s.scalar(select(table))
                self.assertIsNone(result)

            for table in many_to_one_tables:
                result = s.scalar(select(table))
                self.assertIsNotNone(result)

    def test_get_message_with_pk(self) -> None:
        remote_jid = JID.from_string("remote1@jid.org")

        m = mod.Moderation(
            account_="testacc1",
            remote_jid_=remote_jid,
            occupant_=None,
            stanza_id="stanzaid123",
            by=None,
            reason=None,
            timestamp=utc_now(),
        )
        self._archive.insert_object(m)

        error = mod.MessageError(
            account_="testacc1",
            remote_jid_=remote_jid,
            message_id="123",
            by=None,
            type="modify",
            text="text",
            condition="somecond",
            condition_text="somecondtext",
            timestamp=utc_now(),
        )

        self._archive.insert_object(error)

        reaction = mod.Reaction(
            account_=self._account,
            remote_jid_=remote_jid,
            occupant_=None,
            id="stanzaid123",
            direction=ChatDirection.INCOMING,
            emojis="😁️;😘️;😇️",
            timestamp=utc_now(),
        )

        self._archive.upsert_row2(reaction, return_full=False)

        marker = mod.DisplayedMarker(
            account_=self._account,
            remote_jid_=remote_jid,
            occupant_=None,
            id="stanzaid123",
            timestamp=utc_now(),
        )

        self._archive.insert_object(marker)

        receipt = mod.Receipt(
            account_=self._account,
            remote_jid_=remote_jid,
            id="123",
            timestamp=utc_now(),
        )

        self._archive.insert_object(receipt)

        retraction_data = mod.Retraction(
            account_=self._account,
            remote_jid_=remote_jid,
            occupant_=None,
            id="stanzaid123",
            direction=ChatDirection.INCOMING,
            timestamp=utc_now(),
        )

        self._archive.insert_row(retraction_data)

        m = mod.Message(
            account_="testacc1",
            remote_jid_=remote_jid,
            resource="test",
            type=MessageType.GROUPCHAT,
            direction=ChatDirection.INCOMING,
            timestamp=utc_now(),
            state=MessageState.ACKNOWLEDGED,
            id="123",
            stanza_id="stanzaid123",
            text="testmessage",
            security_label_=mod.SecurityLabel(
                account_="testacc1",
                remote_jid_=remote_jid,
                label_hash="hash1",
                displaymarking="dm",
                bgcolor="red",
                fgcolor="blue",
                updated_at=utc_now(),
            ),
            thread_id_="testthreadid",
            occupant_=mod.Occupant(
                account_="testacc1",
                remote_jid_=remote_jid,
                id="someoccid",
                real_remote_jid_=JID.from_string("real@remote.jid"),
                nickname="peter",
                avatar_sha="sha1",
                updated_at=utc_now(),
            ),
            encryption_=mod.Encryption(protocol="OMEMO", key="somekey", trust=1),
            call=mod.Call(sid="somesid", state=1),
            oob=[mod.OOB(url="someurl", description="desc")],
            og=[mod.OpenGraph(about="about", title="title")],
            reply=mod.Reply(id="132", to=None),
        )
        m_pk = self._archive.insert_object(m)

        # Correction

        m = mod.Message(
            account_="testacc1",
            remote_jid_=remote_jid,
            resource="test",
            type=MessageType.GROUPCHAT,
            direction=ChatDirection.INCOMING,
            timestamp=utc_now(),
            state=MessageState.ACKNOWLEDGED,
            id="124",
            stanza_id="stanzaid124",
            correction_id="123",
            text="corrected message",
            security_label_=mod.SecurityLabel(
                account_="testacc1",
                remote_jid_=remote_jid,
                label_hash="hash2",
                displaymarking="dm",
                bgcolor="red",
                fgcolor="blue",
                updated_at=utc_now(),
            ),
            thread_id_="testthreadid",
            occupant_=mod.Occupant(
                account_="testacc1",
                remote_jid_=remote_jid,
                id="someoccid",
                real_remote_jid_=JID.from_string("real@remote.jid"),
                nickname="peter",
                avatar_sha="sha1",
                updated_at=utc_now(),
            ),
            encryption_=mod.Encryption(protocol="OMEMO", key="some_cor_key", trust=1),
            call=mod.Call(sid="some_cor_sid", state=1),
            oob=[mod.OOB(url="some_cor_url", description="desc")],
            og=[mod.OpenGraph(about="about_cor", title="title")],
            reply=mod.Reply(id="133", to=None),
        )

        self._archive.insert_object(m)

        m = mod.Moderation(
            account_="testacc1",
            remote_jid_=remote_jid,
            occupant_=None,
            stanza_id="stanzaid124",
            by=None,
            reason=None,
            timestamp=utc_now(),
        )
        self._archive.insert_object(m)

        error = mod.MessageError(
            account_="testacc1",
            remote_jid_=remote_jid,
            message_id="124",
            by=None,
            type="modify",
            text="cor_text",
            condition="somecond",
            condition_text="somecondtext",
            timestamp=utc_now(),
        )

        self._archive.insert_object(error)

        reaction = mod.Reaction(
            account_=self._account,
            remote_jid_=remote_jid,
            occupant_=None,
            id="stanzaid124",
            direction=ChatDirection.INCOMING,
            emojis="😁️",
            timestamp=utc_now(),
        )

        self._archive.upsert_row2(reaction, return_full=False)

        marker = mod.DisplayedMarker(
            account_=self._account,
            remote_jid_=remote_jid,
            occupant_=None,
            id="stanzaid124",
            timestamp=utc_now(),
        )

        self._archive.insert_object(marker)

        receipt = mod.Receipt(
            account_=self._account,
            remote_jid_=remote_jid,
            id="124",
            timestamp=utc_now(),
        )

        self._archive.insert_object(receipt)

        retraction_data = mod.Retraction(
            account_=self._account,
            remote_jid_=remote_jid,
            occupant_=None,
            id="stanzaid124",
            direction=ChatDirection.INCOMING,
            timestamp=utc_now(),
        )

        self._archive.insert_row(retraction_data)

        message = self._archive.get_message_with_pk(
            m_pk, options=[selectinload(mod.Message.markers)]
        )
        assert message is not None
        self.assertIsNotNone(message.account)
        self.assertIsNotNone(message.remote)
        self.assertIsNotNone(message.thread)
        self.assertIsNotNone(message.occupant)
        self.assertIsNotNone(message.encryption)
        self.assertIsNotNone(message.security_label)
        self.assertIsNotNone(message.receipt)
        self.assertIsNotNone(message.retraction)
        self.assertIsNotNone(message.moderation)
        self.assertIsNotNone(message.error)
        self.assertIsNotNone(message.call)
        self.assertIsNotNone(message.reply)
        self.assertEqual(len(message.oob), 1)
        self.assertEqual(len(message.reactions), 1)
        # self.assertEqual(len(message.filetransfers), 1)
        self.assertEqual(len(message.og), 1)
        self.assertEqual(len(message.markers), 1)
        self.assertEqual(len(message.corrections), 1)

        # Now check if the correction also has all the relationships loaded
        message = message.corrections[0]

        self.assertIsNotNone(message.account)
        self.assertIsNotNone(message.remote)
        self.assertIsNotNone(message.thread)
        self.assertIsNotNone(message.occupant)
        self.assertIsNotNone(message.encryption)
        assert message.encryption is not None
        self.assertEqual(message.encryption.key, "some_cor_key")
        self.assertIsNotNone(message.security_label)
        assert message.security_label is not None
        self.assertEqual(message.security_label.label_hash, "hash2")
        self.assertIsNotNone(message.receipt)
        self.assertIsNotNone(message.retraction)
        self.assertIsNotNone(message.moderation)
        self.assertIsNotNone(message.error)
        self.assertIsNotNone(message.call)
        self.assertIsNotNone(message.reply)
        self.assertEqual(len(message.oob), 1)
        self.assertEqual(len(message.reactions), 1)
        self.assertEqual(message.reactions[0].emojis, "😁️")
        # self.assertEqual(len(message.filetransfers), 1)
        self.assertEqual(len(message.og), 1)
        self.assertEqual(len(message.markers), 1)

    def test_check_if_stanza_id_exists(self) -> None:
        remote_jid = JID.from_string("remote1@jid.org")
        m = mod.Message(
            account_="testacc1",
            remote_jid_=remote_jid,
            resource="test",
            type=MessageType.CHAT,
            direction=ChatDirection.INCOMING,
            timestamp=utc_now(),
            state=MessageState.ACKNOWLEDGED,
            id="123",
            stanza_id="stanzaid123",
            text="testmessage",
        )
        self._archive.insert_object(m)

        result = self._archive.check_if_stanza_id_exists(
            "testacc1", remote_jid, "stanzaid123"
        )
        self.assertTrue(result)

        result = self._archive.check_if_stanza_id_exists("testacc1", remote_jid, "xxx")
        self.assertFalse(result)

    def test_check_if_message_id_exists(self) -> None:
        remote_jid = JID.from_string("remote1@jid.org")
        m = mod.Message(
            account_="testacc1",
            remote_jid_=remote_jid,
            resource="test",
            type=MessageType.CHAT,
            direction=ChatDirection.INCOMING,
            timestamp=utc_now(),
            state=MessageState.ACKNOWLEDGED,
            id="123",
            stanza_id="stanzaid123",
            text="testmessage",
        )
        self._archive.insert_object(m)

        result = self._archive.check_if_message_id_exists("testacc1", remote_jid, "123")
        self.assertTrue(result)

        result = self._archive.check_if_message_id_exists("testacc1", remote_jid, "xxx")
        self.assertFalse(result)

    def test_block_occupants(self) -> None:
        remote_jid = JID.from_string("remote1@jid.org")
        pks: list[int] = []

        for i in range(10):
            occupant = mod.Occupant(
                account_="testacc1",
                remote_jid_=remote_jid,
                id=f"occupant{i}",
                nickname=f"nickname{i}",
                avatar_sha="sha1",
                blocked=i > 4,
                updated_at=utc_now(),
            )
            pk = self._archive.upsert_row(occupant)
            if i > 4:
                pks.append(pk)

        occupant = mod.Occupant(
            account_="testacc2",
            remote_jid_=remote_jid,
            id="occupant11",
            nickname="nickname11",
            avatar_sha="sha1",
            blocked=True,
            updated_at=utc_now(),
        )
        self._archive.upsert_row(occupant)

        occupants = self._archive.get_blocked_occupants("testacc1")
        res = [o.pk for o in occupants]
        self.assertEqual(res, pks)
        self.assertEqual(len(res), 5)

        # This should not raise an exception because the remote table
        # is joined when queried via get_blocked_occupants()
        for o in occupants:
            o.remote.jid  # noqa: B018

        occupants = self._archive.get_blocked_occupants("testacc2")
        self.assertEqual(len(occupants), 1)

        affected = self._archive.set_block_occupant("testacc1", None, [], True)
        occupants = self._archive.get_blocked_occupants(self._account)
        self.assertEqual(len(occupants), 10)
        self.assertEqual(affected, 5)

        occupant_ids = ["occupant1", "occupant2", "occupant3"]

        affected = self._archive.set_block_occupant(
            "testacc1", None, occupant_ids, False
        )
        occupants = self._archive.get_blocked_occupants(self._account)
        self.assertEqual(len(occupants), 7)
        self.assertEqual(affected, 3)

        affected = self._archive.set_block_occupant("testacc1", remote_jid, [], True)
        occupants = self._archive.get_blocked_occupants(self._account)
        self.assertEqual(len(occupants), 10)
        self.assertEqual(affected, 3)

        occupants = self._archive.get_blocked_occupants("testacc2")
        self.assertEqual(len(occupants), 1)

    def test_get_last_display_markers(self) -> None:
        remote_jid1 = JID.from_string("remote1@jid.org")
        remote_jid2 = JID.from_string("remote2@jid.org")

        for i in range(3):
            occupant = mod.Occupant(
                account_="testacc1",
                remote_jid_=remote_jid1,
                id=f"occupant{i}",
                nickname=f"nickname{i}",
                avatar_sha="sha1",
                blocked=False,
                updated_at=utc_now(),
            )

            for i in range(3):
                marker = mod.DisplayedMarker(
                    account_="testacc1",
                    remote_jid_=remote_jid1,
                    occupant_=occupant,
                    id=f"message{i}",
                    timestamp=utc_now() + timedelta(seconds=i),
                )
                self._archive.insert_object(marker)

                # Same markers but from different account received
                marker = mod.DisplayedMarker(
                    account_="testacc2",
                    remote_jid_=remote_jid1,
                    occupant_=occupant,
                    id=f"message{i}",
                    timestamp=utc_now() + timedelta(seconds=i),
                )
                self._archive.insert_object(marker)

            # Marker from different JID
            marker = mod.DisplayedMarker(
                account_="testacc1",
                remote_jid_=remote_jid2,
                occupant_=None,
                id="message99",
                timestamp=utc_now(),
            )
            self._archive.insert_object(marker)

        result = self._archive.get_last_display_markers("testacc1", remote_jid1)

        self.assertEqual(len(result), 3)

        res0, res1, res2 = result
        assert res0.occupant is not None
        assert res1.occupant is not None
        assert res2.occupant is not None

        self.assertEqual(res0.occupant.id, "occupant0")
        self.assertEqual(res0.id, "message2")

        self.assertEqual(res1.occupant.id, "occupant1")
        self.assertEqual(res1.id, "message2")

        self.assertEqual(res2.occupant.id, "occupant2")
        self.assertEqual(res2.id, "message2")

    def test_get_occupant_by_jids(self) -> None:
        remote_jid1 = JID.from_string("remote1@jid.org")

        real_remote_jid1 = JID.from_string("real1@remote.jid")
        real_remote_jid3 = JID.from_string("real3@remote.jid")

        occupant1 = mod.Occupant(
            account_=self._account,
            remote_jid_=remote_jid1,
            id="1",
            real_remote_jid_=real_remote_jid1,
            nickname="peter1",
            avatar_sha="sha1",
            updated_at=datetime.fromtimestamp(101, UTC),
        )

        occupant2 = mod.Occupant(
            account_=self._account,
            remote_jid_=remote_jid1,
            id="2",
            real_remote_jid_=real_remote_jid1,
            nickname="peter2",
            avatar_sha="sha1",
            updated_at=datetime.fromtimestamp(100, UTC),
        )

        occupant3 = mod.Occupant(
            account_=self._account,
            remote_jid_=remote_jid1,
            id="3",
            real_remote_jid_=real_remote_jid3,
            nickname="susi",
            avatar_sha="sha1",
            updated_at=datetime.fromtimestamp(102, UTC),
        )

        self._archive.upsert_row(occupant1)
        self._archive.upsert_row(occupant2)
        self._archive.upsert_row(occupant3)

        occupants = self._archive.get_occupant_by_jids(
            self._account,
            remote_jid1,
            [real_remote_jid1, real_remote_jid3],
            max_age=timedelta(minutes=60),
        )
        self.assertEqual(len(occupants), 2)

        occupant1 = occupants[real_remote_jid1]
        occupant3 = occupants[real_remote_jid3]

        assert occupant1.real_remote is not None
        assert occupant3.real_remote is not None

        self.assertEqual(occupant1.id, "1")
        self.assertEqual(occupant1.real_remote.jid, real_remote_jid1)
        self.assertEqual(occupant1.nickname, "peter1")
        self.assertEqual(occupant1.updated_at, datetime.fromtimestamp(101, UTC))

        self.assertEqual(occupant3.id, "3")
        self.assertEqual(occupant3.real_remote.jid, real_remote_jid3)
        self.assertEqual(occupant3.nickname, "susi")


if __name__ == "__main__":
    unittest.main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import Gtk

from gajim.gtk.avatar_selector import AvatarSelector
from gajim.gtk.window import GajimAppWindow

from . import util

DEFAULT_IMAGE_FILE_PATH = (
    util.get_gajim_dir() / "data/icons/hicolor/96x96/apps/gajim.png"
)


class TestAvatarSelector(GajimAppWindow):
    def __init__(self) -> None:
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=800,
            default_height=800,
            add_window_padding=True,
            header_bar=True,
        )

        box = Gtk.Box(halign=Gtk.Align.CENTER, valign=Gtk.Align.CENTER, hexpand=True)
        self.set_child(box)

        avatar_selector = AvatarSelector()
        avatar_selector.prepare_crop_area(str(DEFAULT_IMAGE_FILE_PATH))
        box.append(avatar_selector)

    def _cleanup(self) -> None:
        pass


util.init_settings()

window = TestAvatarSelector()
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import unittest
from datetime import datetime
from datetime import UTC

from nbxmpp.protocol import JID

from gajim.common import app
from gajim.common.helpers import get_uuid
from gajim.common.settings import Settings
from gajim.common.storage.archive.const import ChatDirection
from gajim.common.storage.archive.const import MessageState
from gajim.common.storage.archive.const import MessageType
from gajim.common.storage.archive.models import Message
from gajim.common.storage.archive.models import Occupant
from gajim.common.storage.archive.models import OOB
from gajim.common.storage.archive.storage import MessageArchiveStorage


class CorrectionsTest(unittest.TestCase):
    def setUp(self) -> None:
        self._archive = MessageArchiveStorage(in_memory=True)
        self._archive.init()

        self._account = "testacc1"
        self._account_jid = JID.from_string("user@domain.org")
        self._remote_jid = JID.from_string("remote@jid.org")
        self._occupant_id = "occupantid1"
        self._init_settings()

    def tearDown(self) -> None:
        self._archive.shutdown()

    def _init_settings(self) -> None:
        app.settings = Settings(in_memory=True)
        app.settings.init()
        app.settings.add_account("testacc1")
        app.settings.set_account_setting("testacc1", "address", "user@domain.org")

        app.settings.add_account("testacc2")
        app.settings.set_account_setting("testacc2", "address", "user2@domain.org")

    def _create_base_occupant(self, occupant_id: str) -> Occupant:
        return Occupant(
            account_=self._account,
            remote_jid_=self._remote_jid,
            id=occupant_id,
            nickname="peter",
            updated_at=datetime.fromtimestamp(50, UTC),
        )

    def _create_base_message(self) -> Message:
        return Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            type=MessageType.GROUPCHAT,
            direction=ChatDirection.INCOMING,
            state=MessageState.ACKNOWLEDGED,
            timestamp=datetime.fromtimestamp(0, UTC),
            resource="res1",
            text="Some Message",
            id="messageid1",
            stanza_id=get_uuid(),
            correction_id=None,
            occupant_=self._create_base_occupant("occupantid1"),
            oob=[OOB(url="https://www.test.com", description="desc")],
        )

    def test_join_correction_groupchat_occupant_id(self) -> None:
        # Order corrections by timestamp
        # Join with occupant id

        m = self._create_base_message()
        message_pk = self._archive.insert_object(m)

        # Insert in not ascending order to test ordering

        m = self._create_base_message()
        m.text = "second correction"
        m.id = "messageid3"
        m.timestamp = datetime.fromtimestamp(2, UTC)
        m.correction_id = "messageid1"
        assert m.oob
        m.oob[0].description = "second corrected desc"

        self._archive.insert_object(m)

        # Change resource, but leave occupant id the same
        # this message should be still joined

        m = self._create_base_message()
        m.text = "first correction"
        m.resource = "otherres"
        m.id = "messageid2"
        m.timestamp = datetime.fromtimestamp(1, UTC)
        m.correction_id = "messageid1"

        self._archive.insert_object(m)

        # Add correction from unrelated occupant, should not be joined

        m = self._create_base_message()
        m.occupant_ = self._create_base_occupant("occupantid2")
        m.text = "third correction"
        m.id = "messageid4"
        m.timestamp = datetime.fromtimestamp(3, UTC)
        m.correction_id = "messageid1"

        self._archive.insert_object(m)

        # Add correction with no occupant id, should not be joined

        m = self._create_base_message()
        m.occupant_ = None
        m.text = "third correction"
        m.id = "messageid5"
        m.timestamp = datetime.fromtimestamp(4, UTC)
        m.correction_id = "messageid1"

        self._archive.insert_object(m)

        message = self._archive.get_message_with_pk(message_pk)

        assert message is not None
        assert message.corrections

        self.assertEqual(len(message.corrections), 2)
        self.assertEqual(message.corrections[0].text, "first correction")
        self.assertEqual(message.corrections[1].text, "second correction")
        assert message.corrections[1].oob is not None
        self.assertEqual(
            message.corrections[1].oob[0].description, "second corrected desc"
        )

    def test_join_correction_groupchat_resource(self) -> None:
        # Join only with resource, when occupant id is not supported

        m = self._create_base_message()
        m.occupant_ = None
        message_pk = self._archive.insert_object(m)

        m = self._create_base_message()
        m.occupant_ = None
        m.text = "first correction"
        m.id = "messageid2"
        m.timestamp = datetime.fromtimestamp(1, UTC)
        m.correction_id = "messageid1"

        self._archive.insert_object(m)

        # Add correction from unrelated resource, should not be joined

        m = self._create_base_message()
        m.occupant_ = None
        m.resource = "unrelatedres"
        m.text = "second correction"
        m.id = "messageid3"
        m.timestamp = datetime.fromtimestamp(2, UTC)
        m.correction_id = "messageid1"

        self._archive.insert_object(m)

        message = self._archive.get_message_with_pk(message_pk)

        assert message is not None
        assert message.corrections

        self.assertEqual(len(message.corrections), 1)
        self.assertEqual(message.corrections[0].text, "first correction")

    def test_join_correction_single_chat(self) -> None:
        # Order corrections by timestamp
        # Join with occupant id

        m = self._create_base_message()
        m.type = MessageType.CHAT
        message_pk = self._archive.insert_object(m)

        m = self._create_base_message()
        m.type = MessageType.CHAT
        m.text = "first correction"
        m.id = "messageid2"
        m.timestamp = datetime.fromtimestamp(1, UTC)
        m.correction_id = "messageid1"

        self._archive.insert_object(m)

        # Change resource, should still be joined because
        # resource does not matter in single chat

        m = self._create_base_message()
        m.type = MessageType.CHAT
        m.text = "second correction"
        m.resource = "otherres"
        m.id = "messageid3"
        m.timestamp = datetime.fromtimestamp(2, UTC)
        m.correction_id = "messageid1"

        self._archive.insert_object(m)

        # Change remote jid, should not be joined

        m = self._create_base_message()
        m.type = MessageType.CHAT
        m.remote_jid_ = JID.from_string("other@remote.jid")
        m.text = "third correction"
        m.id = "messageid4"
        m.timestamp = datetime.fromtimestamp(3, UTC)
        m.correction_id = "messageid1"

        self._archive.insert_object(m)

        # Change account, should not be joined

        m = self._create_base_message()
        m.type = MessageType.CHAT
        m.account_ = "testacc2"
        m.text = "fourth correction"
        m.id = "messageid5"
        m.timestamp = datetime.fromtimestamp(4, UTC)
        m.correction_id = "messageid1"

        self._archive.insert_object(m)

        # Change direction, should not be joined

        m = self._create_base_message()
        m.type = MessageType.CHAT
        m.direction = ChatDirection.OUTGOING
        m.text = "fifth correction"
        m.id = "messageid6"
        m.timestamp = datetime.fromtimestamp(5, UTC)
        m.correction_id = "messageid1"

        self._archive.insert_object(m)

        message = self._archive.get_message_with_pk(message_pk)

        assert message is not None
        assert message.corrections

        self.assertEqual(len(message.corrections), 2)
        self.assertEqual(message.corrections[0].text, "first correction")
        self.assertEqual(message.corrections[1].text, "second correction")


if __name__ == "__main__":
    unittest.main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import unittest
from datetime import datetime
from datetime import UTC

from nbxmpp.protocol import JID
from sqlalchemy.exc import IntegrityError

from gajim.common import app
from gajim.common.helpers import get_uuid
from gajim.common.settings import Settings
from gajim.common.storage.archive.const import ChatDirection
from gajim.common.storage.archive.const import MessageState
from gajim.common.storage.archive.const import MessageType
from gajim.common.storage.archive.models import Message
from gajim.common.storage.archive.models import Receipt
from gajim.common.storage.archive.storage import MessageArchiveStorage


class ReceiptTest(unittest.TestCase):
    def setUp(self) -> None:
        self._archive = MessageArchiveStorage(in_memory=True)
        self._archive.init()

        self._account = "testacc1"
        self._account_jid = JID.from_string("user@domain.org")
        self._remote_jid = JID.from_string("remote@jid.org")
        self._init_settings()

    def tearDown(self) -> None:
        self._archive.shutdown()

    def _init_settings(self) -> None:
        app.settings = Settings(in_memory=True)
        app.settings.init()
        app.settings.add_account("testacc1")
        app.settings.set_account_setting("testacc1", "address", "user@domain.org")

    def test_receipt_join(self):
        receipt_data1 = Receipt(
            account_=self._account,
            remote_jid_=self._remote_jid,
            id="messageid1",
            timestamp=datetime.fromtimestamp(1, UTC),
        )

        receipt_data2 = Receipt(
            account_=self._account,
            remote_jid_=self._remote_jid,
            id="messageid1",
            timestamp=datetime.fromtimestamp(3, UTC),
        )

        self._archive.insert_object(receipt_data1)

        with self.assertRaises(IntegrityError):
            self._archive.insert_object(receipt_data2, ignore_on_conflict=False)

        message_data = Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            type=MessageType.CHAT,
            direction=ChatDirection.OUTGOING,
            timestamp=datetime.now(UTC),
            state=MessageState.ACKNOWLEDGED,
            resource="res",
            text="Some Message",
            id="messageid1",
            stanza_id=get_uuid(),
            occupant_=None,
        )

        pk = self._archive.insert_object(message_data)

        message = self._archive.get_message_with_pk(pk)

        assert message is not None
        assert message.receipt is not None

        self.assertIsNotNone(message.receipt)

        self.assertEqual(message.receipt.timestamp, datetime.fromtimestamp(1, UTC))


if __name__ == "__main__":
    unittest.main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from unittest.mock import MagicMock

from gi.repository import GLib
from gi.repository import Gtk
from nbxmpp import JID
from nbxmpp.const import Chatstate

from gajim.common.modules.contacts import BareContact
from gajim.common.modules.contacts import GroupchatContact
from gajim.common.modules.contacts import GroupchatParticipant
from gajim.common.util.user_strings import chatstate_to_string

from gajim.gtk.chat_state_indicator import ChatStateIndicator
from gajim.gtk.window import GajimAppWindow

from . import util

ACCOUNT = "account"


class TestChatStateIndicator(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=600,
            default_height=600,
            add_window_padding=True,
            header_bar=True,
        )

        box = Gtk.Box(
            halign=Gtk.Align.CENTER,
            valign=Gtk.Align.CENTER,
            hexpand=True,
            orientation=Gtk.Orientation.VERTICAL,
            spacing=12,
        )
        self.set_child(box)

        self._chat_state_indicator = ChatStateIndicator()
        box.append(self._chat_state_indicator)

        self._participants_count = 3
        contact = self._get_groupchat_contact()
        self._chat_state_indicator.switch_contact(contact)

        GLib.timeout_add_seconds(3, self._decrease_participants_count)
        GLib.timeout_add_seconds(3, self._chat_state_indicator.switch_contact, contact)

        GLib.timeout_add_seconds(6, self._decrease_participants_count)
        GLib.timeout_add_seconds(6, self._chat_state_indicator.switch_contact, contact)

        contact = self._get_bare_contact()
        GLib.timeout_add_seconds(9, self._chat_state_indicator.switch_contact, contact)

        contact = self._get_bare_contact(composing=False)
        GLib.timeout_add_seconds(12, self._chat_state_indicator.switch_contact, contact)

    def _decrease_participants_count(self) -> None:
        self._participants_count -= 1

    def _get_bare_contact(self, composing: bool = True) -> BareContact:
        contact = MagicMock(spec_set=BareContact)
        contact.connect = MagicMock()
        contact.account = ACCOUNT
        contact.jid = JID.from_string("user@example.org")
        contact.name = "Test Contact"
        if composing:
            contact.chatstate = Chatstate.COMPOSING
            contact.chatstate_string = chatstate_to_string(Chatstate.COMPOSING)
        else:
            contact.chatstate = Chatstate.PAUSED
            contact.chatstate_string = chatstate_to_string(Chatstate.PAUSED)
        return contact

    def _get_groupchat_contact(self) -> GroupchatContact:
        contact = MagicMock(spec_set=GroupchatContact)
        contact.connect = MagicMock()
        contact.account = ACCOUNT
        contact.jid = JID.from_string("groupchat@conference.example.org")
        contact.name = "Test Group Chat"
        contact.is_groupchat = True
        contact.get_composers = MagicMock(side_effect=self._get_composers)
        return contact

    def _get_groupchat_participant(self, name: str) -> GroupchatParticipant:
        contact = MagicMock(spec_set=GroupchatParticipant)
        contact.connect = MagicMock()
        contact.account = ACCOUNT
        contact.jid = JID.from_string(f"groupchat@conferece.example.org/{name}")
        contact.name = name
        return contact

    def _get_composers(self) -> list[GroupchatParticipant]:
        participants: list[GroupchatParticipant] = []
        for name in [f"user{count + 1}" for count in range(self._participants_count)]:
            participants.append(self._get_groupchat_participant(name))
        return participants

    def _cleanup(self) -> None:
        pass


util.init_settings()

window = TestChatStateIndicator()
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import unittest
from datetime import datetime
from datetime import UTC

from nbxmpp.protocol import JID
from sqlalchemy import select

from gajim.common import app
from gajim.common.settings import Settings
from gajim.common.storage.archive.const import ChatDirection
from gajim.common.storage.archive.const import MessageState
from gajim.common.storage.archive.const import MessageType
from gajim.common.storage.archive.models import Message
from gajim.common.storage.archive.models import Occupant
from gajim.common.storage.archive.storage import MessageArchiveStorage


class OccupantTest(unittest.TestCase):
    def setUp(self) -> None:
        self._archive = MessageArchiveStorage(in_memory=True)
        self._archive.init()

        self._account = "testacc1"
        self._account_jid = JID.from_string("user@domain.org")
        self._remote_jid = JID.from_string("remote@jid.org")
        self._occupant_id = "occupantid1"
        self._init_settings()

    def tearDown(self) -> None:
        self._archive.shutdown()

    def _init_settings(self) -> None:
        app.settings = Settings(in_memory=True)
        app.settings.init()
        app.settings.add_account("testacc1")
        app.settings.set_account_setting("testacc1", "address", "user@domain.org")

    def test_insert_occupant(self) -> None:
        occupant_data = Occupant(
            account_=self._account,
            remote_jid_=self._remote_jid,
            id="someid",
            nickname="peter",
            updated_at=datetime.fromtimestamp(50, UTC),
        )

        pk = self._archive.upsert_row(occupant_data)

        with self._archive.get_session() as s:
            occupant = s.scalar(select(Occupant).where(Occupant.pk == pk))
        assert occupant is not None

        self.assertEqual(occupant.id, "someid")
        self.assertEqual(occupant.real_remote, None)
        self.assertEqual(occupant.nickname, "peter")
        self.assertEqual(occupant.avatar_sha, None)
        self.assertEqual(occupant.updated_at, datetime.fromtimestamp(50, UTC))

        occupant_data = Occupant(
            account_=self._account,
            remote_jid_=self._remote_jid,
            id="someid",
            real_remote_jid_=JID.from_string("real@remote.jid"),
            nickname="peter",
            avatar_sha="sha1",
            updated_at=datetime.fromtimestamp(100, UTC),
        )

        pk = self._archive.upsert_row(occupant_data)

        with self._archive.get_session() as s:
            occupant = s.scalar(select(Occupant).where(Occupant.pk == pk))
        assert occupant is not None
        assert occupant.real_remote is not None

        self.assertEqual(occupant.id, "someid")
        self.assertEqual(occupant.real_remote.jid, "real@remote.jid")
        self.assertIsInstance(occupant.real_remote.jid, JID)
        self.assertEqual(occupant.nickname, "peter")
        self.assertEqual(occupant.avatar_sha, "sha1")
        self.assertEqual(occupant.updated_at, datetime.fromtimestamp(100, UTC))

        # Update with old data should be ignored

        occupant_data = Occupant(
            account_=self._account,
            remote_jid_=self._remote_jid,
            id="someid",
            real_remote_jid_=JID.from_string("real2@remote.jid"),
            nickname="peter2",
            avatar_sha="sha2",
            updated_at=datetime.fromtimestamp(50, UTC),
        )

        pk = self._archive.upsert_row(occupant_data)
        self.assertEqual(pk, 1)

        with self._archive.get_session() as s:
            occupant = s.scalar(select(Occupant).where(Occupant.pk == pk))
        assert occupant is not None
        assert occupant.real_remote is not None

        self.assertEqual(occupant.id, "someid")
        self.assertEqual(occupant.real_remote.jid, "real@remote.jid")
        self.assertIsInstance(occupant.real_remote.jid, JID)
        self.assertEqual(occupant.nickname, "peter")
        self.assertEqual(occupant.avatar_sha, "sha1")
        self.assertEqual(occupant.updated_at, datetime.fromtimestamp(100, UTC))

        # Update with new data
        # Remote JID should not be overwritten

        occupant_data = Occupant(
            account_=self._account,
            remote_jid_=self._remote_jid,
            id="someid",
            real_remote_jid_=JID.from_string("real2@remote.jid"),
            nickname="peter2",
            avatar_sha="sha2",
            updated_at=datetime.fromtimestamp(200, UTC),
        )

        pk = self._archive.upsert_row(occupant_data)
        self.assertEqual(pk, 1)

        with self._archive.get_session() as s:
            occupant = s.scalar(select(Occupant).where(Occupant.pk == pk))
        assert occupant is not None
        assert occupant.real_remote is not None

        self.assertEqual(occupant.id, "someid")
        self.assertEqual(occupant.real_remote.jid, "real@remote.jid")
        self.assertIsInstance(occupant.real_remote.jid, JID)
        self.assertEqual(occupant.nickname, "peter2")
        self.assertEqual(occupant.avatar_sha, "sha2")
        self.assertEqual(occupant.updated_at, datetime.fromtimestamp(200, UTC))

        # Set avatar_sha None
        # Passing None to for nickname should be ignored

        occupant_data = Occupant(
            account_=self._account,
            remote_jid_=self._remote_jid,
            id="someid",
            nickname=None,
            avatar_sha=None,
            updated_at=datetime.fromtimestamp(300, UTC),
        )

        pk = self._archive.upsert_row(occupant_data)
        self.assertEqual(pk, 1)

        with self._archive.get_session() as s:
            occupant = s.scalar(select(Occupant).where(Occupant.pk == pk))
        assert occupant is not None
        assert occupant.real_remote is not None

        self.assertEqual(occupant.id, "someid")
        self.assertEqual(occupant.real_remote.jid, "real@remote.jid")
        self.assertIsInstance(occupant.real_remote.jid, JID)
        self.assertEqual(occupant.nickname, "peter2")
        self.assertEqual(occupant.avatar_sha, None)
        self.assertEqual(occupant.updated_at, datetime.fromtimestamp(300, UTC))

    def test_occupant_join(self) -> None:
        occupant_data = Occupant(
            account_=self._account,
            remote_jid_=self._remote_jid,
            id="someid",
            real_remote_jid_=JID.from_string("real@remote.jid"),
            nickname="peter3",
            avatar_sha="sha1",
            updated_at=datetime.fromtimestamp(300, UTC),
        )

        message_data = Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            resource="someres1",
            type=MessageType.CHAT,
            direction=ChatDirection.INCOMING,
            timestamp=datetime.now(UTC),
            state=MessageState.ACKNOWLEDGED,
            id="1",
            text="message",
            occupant_=occupant_data,
        )

        pk = self._archive.insert_object(message_data)

        message = self._archive.get_message_with_pk(pk)

        assert message is not None
        assert message.occupant is not None
        assert message.occupant.real_remote is not None

        self.assertEqual(message.occupant.nickname, "peter3")
        self.assertEqual(message.occupant.avatar_sha, "sha1")
        self.assertEqual(message.occupant.real_remote.jid, "real@remote.jid")
        self.assertIsInstance(message.occupant.real_remote.jid, JID)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

import os
import sys
from argparse import ArgumentParser
from subprocess import CalledProcessError
from subprocess import check_output
from subprocess import run

if __name__ == "__main__":
    if not os.path.isdir("mac"):
        sys.exit(
            'Unable to find "mac" directory. make sure you run '
            "this script from the project root"
        )

    parser = ArgumentParser(
        description="Create a macOS .app bundle. "
        "Requires PyInstaller and hdiutil (macOS)."
    )
    parser.add_argument("--version", help="version number of the .app bundle")
    args = parser.parse_args()

    if args.version:
        version = args.version
    else:
        try:
            version = check_output(["git", "describe", "--tags"])
            version = version.decode().strip()
        except CalledProcessError:
            version = "unknown"
    dmg_name = f"gajim-{version}.dmg"

    # the .spec has to be in the project root
    run(["cp", "mac/gajim.spec", "gajim.spec"], check=True)
    run(["pyinstaller", "gajim.spec"], check=True)
    # we only want Gajim.app in the dmg
    run(["rm", "-rf", "dist/launch"], check=True)
    run(
        [
            "hdiutil",
            "create",
            "-volname",
            "Gajim",
            "-srcfolder",
            "dist",
            "-ov",
            "-format",
            "UDZO",
            dmg_name,
        ],
        check=True,
    )



from keyring.backend import KeyringBackend

def get_password(service_name: str, username: str) -> str | None: ...
def set_password(service_name: str, username: str, password: str) -> None: ...
def delete_password(service_name: str, username: str) -> None: ...
def recommended(backend: KeyringBackend) -> bool: ...
def get_keyring() -> KeyringBackend: ...



"""
This type stub file was generated by pyright.
"""

from typing import Optional

from dataclasses import dataclass

from winrt import system

@dataclass
class ToastActivatedEventArgs:
    """
    Wrapper over Windows' ToastActivatedEventArgs to fix an issue with reading user input
    """

    arguments: Optional[str] = ...
    inputs: Optional[dict] = ...
    @classmethod
    def fromWinRt(cls, eventArgs: system.Object) -> ToastActivatedEventArgs: ...

class ToastDismissalReason: ...
class ToastDismissedEventArgs: ...
class ToastFailedEventArgs: ...



"""
This type stub file was generated by pyright.
"""

class InvalidImageException(Exception):
    """The image provided was invalid"""

    ...

class ToastNotFoundError(Exception):
    """The toast could not be found"""

    ...

class UnsupportedOSVersionException(ImportError):
    """The operating system version is not supported"""

    ...



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gajim.main import gi_require_versions

gi_require_versions()

from gajim.common import app
from gajim.common.settings import Settings

app.settings = Settings(in_memory=True)
app.settings.init()



"""
This type stub file was generated by pyright.
"""

from typing import Union

from dataclasses import dataclass
from enum import Enum

class AudioSource(Enum):
    """
    Different audios built into Windows
    """

    Default = ...
    IM = ...
    Mail = ...
    Reminder = ...
    SMS = ...
    Alarm = ...
    Alarm2 = ...
    Alarm3 = ...
    Alarm4 = ...
    Alarm5 = ...
    Alarm6 = ...
    Alarm7 = ...
    Alarm8 = ...
    Alarm9 = ...
    Alarm10 = ...
    Call = ...
    Call2 = ...
    Call3 = ...
    Call4 = ...
    Call5 = ...
    Call6 = ...
    Call7 = ...
    Call8 = ...
    Call9 = ...
    Call10 = ...

@dataclass
class ToastAudio:
    """
    Audio configuration in a toast

    :param sound: Selected AudioSource to play
    :type sound: Union[AudioSource, str]
    :param looping: Whether the audio should loop once it ends. Stops abruptly when the notification is dismissed
    :type looping: bool
    :param silent: Silence any audio
    :type silent: bool
    """

    sound: Union[AudioSource, str] = ...
    looping: bool = ...
    silent: bool = ...
    @property
    def sound_value(self) -> str:
        """
        Returns the string value of the selected sound
        """
        ...



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import unittest

from nbxmpp.protocol import JID

from gajim.common import app
from gajim.common.helpers import get_uuid
from gajim.common.settings import Settings
from gajim.common.storage.archive.const import ChatDirection
from gajim.common.storage.archive.const import MessageState
from gajim.common.storage.archive.const import MessageType
from gajim.common.storage.archive.models import Message
from gajim.common.storage.archive.models import Occupant
from gajim.common.storage.archive.models import Reaction
from gajim.common.storage.archive.storage import MessageArchiveStorage
from gajim.common.util.datetime import utc_now

from .util import mk_utc_dt


class ReactionsTest(unittest.TestCase):
    def setUp(self) -> None:
        self._archive = MessageArchiveStorage(in_memory=True)
        self._archive.init()

        self._account = "testacc1"
        self._account_jid = JID.from_string("user@domain.org")
        self._remote_jid = JID.from_string("remote@jid.org")
        self._init_settings()

    def tearDown(self) -> None:
        self._archive.shutdown()

    def _init_settings(self) -> None:
        app.settings = Settings(in_memory=True)
        app.settings.init()
        app.settings.add_account("testacc1")
        app.settings.set_account_setting("testacc1", "address", "user@domain.org")

    def test_reactions_join(self):
        r_data1 = Reaction(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=None,
            id="messageid1",
            direction=ChatDirection.INCOMING,
            emojis="😁️",
            timestamp=mk_utc_dt(10),
        )

        r_data2 = Reaction(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=None,
            id="messageid1",
            direction=ChatDirection.INCOMING,
            emojis="😁️;😘️;😇️",
            timestamp=mk_utc_dt(11),
        )

        r_data3 = Reaction(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=None,
            id="messageid1",
            direction=ChatDirection.INCOMING,
            emojis="😘️",
            timestamp=mk_utc_dt(9),
        )

        r_data4 = Reaction(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=None,
            id="messageid1",
            direction=ChatDirection.OUTGOING,
            emojis="😘️",
            timestamp=mk_utc_dt(9),
        )

        pk = self._archive.upsert_row2(r_data1, return_full=False)
        self.assertIsNotNone(pk)
        pk = self._archive.upsert_row2(r_data2, return_full=False)
        self.assertIsNotNone(pk)
        pk = self._archive.upsert_row2(r_data3, return_full=False)
        self.assertIsNone(pk)
        pk = self._archive.upsert_row2(r_data4, return_full=False)
        self.assertIsNotNone(pk)

        message_data = Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            type=MessageType.CHAT,
            direction=ChatDirection.OUTGOING,
            timestamp=utc_now(),
            state=MessageState.ACKNOWLEDGED,
            resource="res",
            text="Some Message",
            id="messageid1",
            stanza_id=get_uuid(),
            occupant_=None,
        )

        pk = self._archive.insert_object(message_data)

        message = self._archive.get_message_with_pk(pk)

        assert message is not None

        self.assertTrue(len(message.reactions), 2)

        self.assertEqual(message.reactions[0].emojis, "😁️;😘️;😇️")
        self.assertEqual(message.reactions[1].emojis, "😘️")

    def test_reactions_join_groupchat(self):
        occupant_data1 = Occupant(
            account_=self._account,
            remote_jid_=self._remote_jid,
            id="occupantid1",
            nickname="nickname1",
            updated_at=mk_utc_dt(0),
        )

        occupant_data2 = Occupant(
            account_=self._account,
            remote_jid_=self._remote_jid,
            id="occupantid2",
            nickname="nickname2",
            updated_at=mk_utc_dt(0),
        )

        uuid = get_uuid()

        r_data1 = Reaction(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=occupant_data1,
            id=uuid,
            direction=ChatDirection.INCOMING,
            emojis="😁️",
            timestamp=mk_utc_dt(1),
        )

        r_data2 = Reaction(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=occupant_data2,
            id=uuid,
            direction=ChatDirection.INCOMING,
            emojis="😘️",
            timestamp=mk_utc_dt(2),
        )

        r_data3 = Reaction(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=occupant_data1,
            id=uuid,
            direction=ChatDirection.INCOMING,
            emojis="😇️;😁️",
            timestamp=mk_utc_dt(3),
        )

        pk = self._archive.upsert_row2(r_data1, return_full=False)
        self.assertIsNotNone(pk)
        pk = self._archive.upsert_row2(r_data2, return_full=False)
        self.assertIsNotNone(pk)
        pk = self._archive.upsert_row2(r_data3, return_full=False)
        self.assertIsNotNone(pk)

        message_data = Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            type=MessageType.GROUPCHAT,
            direction=ChatDirection.INCOMING,
            timestamp=mk_utc_dt(0),
            state=MessageState.ACKNOWLEDGED,
            resource="res",
            text="Some Message",
            id="messageid99",
            stanza_id=uuid,
            occupant_=None,
        )

        pk = self._archive.insert_object(message_data)

        message = self._archive.get_message_with_pk(pk)

        assert message is not None

        self.assertTrue(len(message.reactions), 2)

        r1, r2 = message.reactions

        assert r1.occupant is not None
        assert r2.occupant is not None

        self.assertEqual(r1.occupant.id, "occupantid1")
        self.assertEqual(r1.timestamp, mk_utc_dt(3))
        self.assertEqual(r1.emojis, "😇️;😁️")
        self.assertEqual(r2.occupant.id, "occupantid2")
        self.assertEqual(r2.timestamp, mk_utc_dt(2))
        self.assertEqual(r2.emojis, "😘️")

    def test_combine_revisions(self):
        # Reactions can land on multiple revisions (corrections) of the
        # message. Check that get_reactions() only returns the reactions
        # on the latest revision.

        occupant_data1 = Occupant(
            account_=self._account,
            remote_jid_=self._remote_jid,
            id="occupantid1",
            nickname="nickname1",
            updated_at=mk_utc_dt(0),
        )

        occupant_data2 = Occupant(
            account_=self._account,
            remote_jid_=self._remote_jid,
            id="occupantid2",
            nickname="nickname2",
            updated_at=mk_utc_dt(0),
        )

        occupant_data3 = Occupant(
            account_=self._account,
            remote_jid_=self._remote_jid,
            id="occupantid3",
            nickname="nickname3",
            updated_at=mk_utc_dt(0),
        )

        uuid1 = get_uuid()
        uuid2 = get_uuid()

        r_data1 = Reaction(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=occupant_data2,
            id=uuid1,
            direction=ChatDirection.INCOMING,
            emojis="😁️",
            timestamp=mk_utc_dt(1),
        )

        r_data2 = Reaction(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=occupant_data2,
            id=uuid2,
            direction=ChatDirection.INCOMING,
            emojis="😘️",
            timestamp=mk_utc_dt(2),
        )

        r_data3 = Reaction(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=occupant_data3,
            id=uuid2,
            direction=ChatDirection.INCOMING,
            emojis="😇️;😁️",
            timestamp=mk_utc_dt(3),
        )

        pk = self._archive.upsert_row2(r_data1, return_full=False)
        self.assertIsNotNone(pk)
        pk = self._archive.upsert_row2(r_data2, return_full=False)
        self.assertIsNotNone(pk)
        pk = self._archive.upsert_row2(r_data3, return_full=False)
        self.assertIsNotNone(pk)

        message_data = Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            type=MessageType.GROUPCHAT,
            direction=ChatDirection.INCOMING,
            timestamp=mk_utc_dt(0),
            state=MessageState.ACKNOWLEDGED,
            resource="res",
            text="Some Message",
            id="messageid99",
            stanza_id=uuid1,
            occupant_=occupant_data1,
        )

        orig_pk = self._archive.insert_object(message_data)

        message_data = Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            type=MessageType.GROUPCHAT,
            direction=ChatDirection.INCOMING,
            timestamp=mk_utc_dt(0),
            state=MessageState.ACKNOWLEDGED,
            resource="res",
            text="Some Message",
            id="messageid100",
            correction_id="messageid99",
            stanza_id=uuid2,
            occupant_=occupant_data1,
        )

        self._archive.insert_object(message_data)

        message = self._archive.get_message_with_pk(orig_pk)
        assert message is not None

        reactions = message.get_reactions()
        self.assertTrue(len(reactions), 2)

        r1, r2 = reactions

        assert r1.occupant is not None
        assert r2.occupant is not None

        self.assertEqual(r1.occupant.id, "occupantid2")
        self.assertEqual(r1.timestamp, mk_utc_dt(2))
        self.assertEqual(r1.emojis, "😘️")
        self.assertEqual(r2.occupant.id, "occupantid3")
        self.assertEqual(r2.timestamp, mk_utc_dt(3))
        self.assertEqual(r2.emojis, "😇️;😁️")


if __name__ == "__main__":
    unittest.main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import logging
import sys
from datetime import datetime
from datetime import UTC


def mk_utc_dt(timestamp: int) -> datetime:
    return datetime.fromtimestamp(timestamp, UTC)


def attach_debug_logger(log_module: str) -> None:
    logger = logging.getLogger(log_module)
    logger.setLevel(logging.DEBUG)
    stream_handler = logging.StreamHandler(sys.stdout)
    logger.addHandler(stream_handler)



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import unittest
from datetime import datetime
from datetime import UTC

from nbxmpp.protocol import JID
from sqlalchemy import select

from gajim.common import app
from gajim.common.helpers import get_uuid
from gajim.common.settings import Settings
from gajim.common.storage.archive.const import ChatDirection
from gajim.common.storage.archive.const import MessageState
from gajim.common.storage.archive.const import MessageType
from gajim.common.storage.archive.models import FileTransfer
from gajim.common.storage.archive.models import FileTransferSource
from gajim.common.storage.archive.models import JinglePub
from gajim.common.storage.archive.models import Message
from gajim.common.storage.archive.models import UrlData
from gajim.common.storage.archive.storage import MessageArchiveStorage


class ForeignKeyTest(unittest.TestCase):
    def setUp(self) -> None:
        self._archive = MessageArchiveStorage(in_memory=True)
        self._archive.init()

        self._account = "testacc1"
        self._account_jid = JID.from_string("user@domain.org")
        self._remote_jid = JID.from_string("remote@jid.org")
        self._occupant_id = "occupantid1"
        self._init_settings()

    def tearDown(self) -> None:
        self._archive.shutdown()

    def _init_settings(self) -> None:
        app.settings = Settings(in_memory=True)
        app.settings.init()
        app.settings.add_account("testacc1")
        app.settings.set_account_setting("testacc1", "address", "user@domain.org")

    def test_filetransfer_join(self) -> None:
        now = datetime.now(UTC)

        source1 = UrlData(
            type="urldata",
            target="http://target",
            scheme_data={"header": "someheader"},
        )

        source2 = JinglePub(
            type="jinglepub",
            id="123",
        )

        ft_data1 = FileTransfer(
            date=now,
            desc="desc",
            hash="abc",
            hash_algo="sha-1",
            height=123,
            length=778272,
            media_type="image/png",
            name="filename1",
            size=6555,
            width=789,
            state=0,
            source=[source1, source2],
        )

        message_data = Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            resource="someres1",
            type=MessageType.CHAT,
            direction=ChatDirection.INCOMING,
            timestamp=now,
            state=MessageState.ACKNOWLEDGED,
            id="1",
            stanza_id=get_uuid(),
            text="message",
            filetransfers=[ft_data1],
        )

        pk = self._archive.insert_object(message_data)

        message = self._archive.get_message_with_pk(pk)

        assert message is not None
        assert message.filetransfers
        ft1 = message.filetransfers[0]
        self.assertEqual(ft1.date, now)
        self.assertEqual(ft1.desc, "desc")
        self.assertEqual(ft1.hash, "abc")
        self.assertEqual(ft1.hash_algo, "sha-1")
        self.assertEqual(ft1.height, 123)
        self.assertEqual(ft1.length, 778272)
        self.assertEqual(ft1.media_type, "image/png")
        self.assertEqual(ft1.name, "filename1")
        self.assertEqual(ft1.size, 6555)
        self.assertEqual(ft1.width, 789)
        self.assertEqual(ft1.state, 0)

        assert ft1.source

        source1 = ft1.source[0]
        source2 = ft1.source[1]
        assert isinstance(source1, UrlData)
        assert isinstance(source2, JinglePub)

        self.assertEqual(source1.type, "urldata")
        self.assertEqual(source1.target, "http://target")
        self.assertEqual(source1.scheme_data, {"header": "someheader"})

        self.assertEqual(source2.type, "jinglepub")
        self.assertEqual(source2.id, "123")

        self._archive.delete_message(message.pk)

        stmts = [
            select(FileTransfer),
            select(UrlData),
            select(FileTransferSource),
        ]

        for stmt in stmts:
            with self._archive.get_session() as s:
                res = s.execute(stmt).one_or_none()
            assert res is None


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations

import typing

import enum

import winrt.system
from winrt.windows.foundation import EventRegistrationToken
from winrt.windows.foundation import TypedEventHandler
from winrt.windows.ui import Color

class UIColorType(enum.IntEnum):
    BACKGROUND = 0
    FOREGROUND = 1
    ACCENT_DARK3 = 2
    ACCENT_DARK2 = 3
    ACCENT_DARK1 = 4
    ACCENT = 5
    ACCENT_LIGHT1 = 6
    ACCENT_LIGHT2 = 7
    ACCENT_LIGHT3 = 8
    COMPLEMENT = 9

@typing.final
class UISettings:
    def get_color_value(self, desired_color: UIColorType, /) -> Color: ...
    def add_color_values_changed(
        self, handler: TypedEventHandler[UISettings, winrt.system.Object], /
    ) -> EventRegistrationToken: ...



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gajim.common import app
from gajim.common.util.muc import message_needs_highlight

app.settings.set("muc_highlight_words", "test;gajim")
JID = "juliet@xmppserver"
NICK = "Romeo"


class HighlightTest(unittest.TestCase):
    """Tests for message_needs_highlight"""

    def test_highlight(self):
        t_text1 = "Romeo: Does this work?"
        t_text2 = "Romeo:Does this work?"
        t_text3 = "Romeo Does this work?"
        t_text4 = "Does this work, romeo?"
        t_text5 = "Does this work,Romeo?"
        t_text6 = "Are you using Gajim?"
        t_text7 = "Did you test this?"
        t_text8 = "Hi romeo"
        t_text9 = "My address is juliet@xmppserver"
        t_text10 = "Romeo, asd"
        t_text11 = "Romeo,"
        t_text12 = "Romeo,hi"
        t_text13 = "@Romeo"
        t_text14 = "#Romeo"

        f_text1 = ""
        f_text2 = "RomeoDoes this work?"
        f_text3 = "nRomeo"
        f_text4 = "nRomeoa"
        f_text5 = "/Romeo"
        f_text6 = "'Romeo"
        f_text_url_1 = "https://romeo.tld"
        f_text_url_2 = "https://romeo.tld message"
        f_text_url_3 = "https://test.tld/where-is-romeo"

        self.assertTrue(message_needs_highlight(t_text1, NICK, JID))
        self.assertTrue(message_needs_highlight(t_text2, NICK, JID))
        self.assertTrue(message_needs_highlight(t_text3, NICK, JID))
        self.assertTrue(message_needs_highlight(t_text4, NICK, JID))
        self.assertTrue(message_needs_highlight(t_text5, NICK, JID))
        self.assertTrue(message_needs_highlight(t_text6, NICK, JID))
        self.assertTrue(message_needs_highlight(t_text7, NICK, JID))
        self.assertTrue(message_needs_highlight(t_text8, NICK, JID))
        self.assertTrue(message_needs_highlight(t_text9, NICK, JID))
        self.assertTrue(message_needs_highlight(t_text10, NICK, JID))
        self.assertTrue(message_needs_highlight(t_text11, NICK, JID))
        self.assertTrue(message_needs_highlight(t_text12, NICK, JID))
        self.assertTrue(message_needs_highlight(t_text13, NICK, JID))
        self.assertTrue(message_needs_highlight(t_text14, NICK, JID))

        self.assertFalse(message_needs_highlight(f_text1, NICK, JID))
        self.assertFalse(message_needs_highlight(f_text2, NICK, JID))
        self.assertFalse(message_needs_highlight(f_text3, NICK, JID))
        self.assertFalse(message_needs_highlight(f_text4, NICK, JID))
        self.assertFalse(message_needs_highlight(f_text5, NICK, JID))
        self.assertFalse(message_needs_highlight(f_text6, NICK, JID))
        self.assertFalse(message_needs_highlight(f_text_url_1, NICK, JID))
        self.assertFalse(message_needs_highlight(f_text_url_2, NICK, JID))
        self.assertFalse(message_needs_highlight(f_text_url_3, NICK, JID))


if __name__ == "__main__":
    unittest.main()



"""
This type stub file was generated by pyright.
"""

import typing

import contextlib

from PIL import Image

@contextlib.contextmanager
def serialized_image(
    image: Image.Image, format: str, extension: str | None = ...
) -> typing.Any:
    """Creates an image file from a :class:`PIL.Image.Image`.

    This function is a context manager that yields a temporary file name. The
    file is removed when the block is exited.

    :param PIL.Image.Image image: The in-memory image.

    :param str format: The format of the image. This format must be handled by
        *Pillow*.

    :param extension: The file extension. This defaults to ``format``
        lowercased.
    :type extensions: str or None
    """
    ...



#!/usr/bin/env python3

# Reads all .ui files and creates builder.pyi
# Execute this script from the repo root dir

import logging
import sys
from io import TextIOWrapper
from pathlib import Path
from xml.etree import ElementTree

logging.basicConfig(level="INFO", format="%(levelname)s: %(message)s")
log = logging.getLogger()

cwd = Path.cwd()

if cwd.name != "gajim":
    sys.exit("Script needs to be executed from gajim repository root directory")

in_path = cwd / "gajim" / "data" / "gui"
out_path = cwd / "gajim" / "gtk" / "builder.pyi"

paths = list(in_path.iterdir())
paths.sort()

IMPORTS = """
from typing import Any
from typing import Literal
from typing import overload

from gi.repository import Adw
from gi.repository import Gtk
from gi.repository import GtkSource


class GajimBuilder:

    def __init__(
        self,
        filename: str | None = None,
        instance: Any = None,
        widgets: list[str] | None = None,
        domain: str | None = None,
        gettext_: Any | None = None
    ) -> None: ...


class Builder(Gtk.Builder):
    ...

"""

CLASS_DEF = "\nclass %s(Builder):"
ATTR = "\n    %s: %s"

GET_BUILDER_OVERLOAD = """
@overload
def get_builder(file_name: Literal['%s'], instance: Any = None, widgets: list[str] = ...) -> %s: ...  # noqa"""  # noqa: E501

GET_BUILDER = """\n\n
def get_builder(file_name: str, instance: Any = None, widgets: list[str] = ...) -> Builder: ..."""  # noqa: E501


class InvalidFile(Exception):
    pass


def make_class_name(path: Path) -> str:
    name = path.name.removesuffix(".ui")
    names = name.split("_")
    names = [name.capitalize() for name in names]
    return "".join(names) + "Builder"


def parse(path: Path, file: TextIOWrapper) -> str:
    log.info("Read %s", path)
    lines: list[str] = []
    tree = ElementTree.parse(path)

    if tree.find("template") is not None:
        raise InvalidFile

    for node in tree.iter(tag="object"):
        id_ = node.attrib.get("id")

        if id_ is None:
            continue
        klass = node.attrib["class"]
        if klass.startswith("GtkSource"):
            klass = f"GtkSource.{klass.removeprefix('GtkSource')}"
        elif klass.startswith("Adw"):
            klass = f"Adw.{klass.removeprefix('Adw')}"
        elif klass.startswith("Atk"):
            klass = f"Atk.{klass.removeprefix('Atk')}"
        else:
            klass = f"Gtk.{klass.removeprefix('Gtk')}"

        lines.append(ATTR % (id_.replace("-", "_"), klass))

    klass_name = make_class_name(path)
    file.write(CLASS_DEF % klass_name)

    if not lines:
        file.write("\n    pass")
    else:
        file.writelines(lines)
    file.write("\n\n")
    return klass_name


builder_names: list[tuple[str, str]] = []

with out_path.open(mode="w", encoding="utf8") as file:
    file.write(IMPORTS)
    for path in paths:
        if path.is_dir():
            continue

        if path.name.endswith("~"):
            continue

        if path.name.startswith("#"):
            continue

        try:
            name = parse(path, file)
        except InvalidFile:
            continue

        builder_names.append((name, path.name))

    for name, file_name in builder_names:
        file.write(GET_BUILDER_OVERLOAD % (file_name, name))
    file.write(GET_BUILDER)
    file.write("\n")



#!/usr/bin/env python

# Copyright 2016 Christoph Reiter
#
# SPDX-License-Identifier: GPL-2.0-or-later

# Creates simple Python .exe launchers for gui and cli apps
#
# ./create_launcher.py "1.4.0" <target-dir>


import os
import shlex
import shutil
import struct
import subprocess
import sys
import tempfile


def build_resource(rc_path: str, out_path: str) -> None:
    """Raises subprocess.CalledProcessError"""

    def is_64bit():
        return struct.calcsize("P") == 8

    subprocess.check_call(
        [
            "windres",
            "-O",
            "coff",
            "-F",
            "pe-x86-64" if is_64bit() else "pe-i386",
            rc_path,
            "-o",
            out_path,
        ]
    )


def get_build_args() -> list[str]:
    python_name = os.path.splitext(os.path.basename(sys.executable))[0]
    python_config = os.path.join(
        os.path.dirname(sys.executable), python_name + "-config"
    )

    cflags = subprocess.check_output(
        [sys.executable, python_config, "--cflags"]
    ).strip()
    libs = subprocess.check_output([sys.executable, python_config, "--libs"]).strip()

    cflags = os.fsdecode(cflags)
    libs = os.fsdecode(libs)
    return shlex.split(cflags) + shlex.split(libs)


def build_exe(
    source_path: str, resource_path: str, out_path: str, *, is_gui: bool
) -> None:
    args = ["gcc", "-s"]
    if is_gui:
        args.append("-mwindows")
    args.extend(["-o", out_path, source_path, resource_path])
    args.extend(get_build_args())
    subprocess.check_call(args)


def get_launcher_code(*, debug: bool) -> str:
    main_code = (
        """
import os
import sys
from ctypes import windll
from pathlib import Path

os.environ['GAJIM_DEBUG'] = '%s'
sys.frozen = True
root_path = Path(sys.executable).parents[1]
windll.kernel32.SetDllDirectoryW(str(root_path / 'bin'))

if __name__ == '__main__':
    import gajim.main
    gajim.main.run()
""".strip()
        % int(debug)
    )

    c_string = ""
    for line in main_code.splitlines():
        c_string += f'        "{line}\\n"\n'

    template = """\
#include "Python.h"
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <shellapi.h>


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow)
{
    int result;
    LPWSTR *szArglist;
    int nArgs;

    szArglist = CommandLineToArgvW(GetCommandLineW(), &nArgs);
    if( NULL == szArglist )
    {
      printf("CommandLineToArgvW failed");
      return 0;
    }

    Py_NoUserSiteDirectory = 1;
    Py_IgnoreEnvironmentFlag = 1;
    Py_DontWriteBytecodeFlag = 1;
    Py_FrozenFlag = 1;
    Py_Initialize();
    PySys_SetArgvEx(__argc, szArglist, 0);
    result = PyRun_SimpleString(
%s
    );
    Py_Finalize();
    return result;
}
    """ % c_string[:-1]

    return template


def get_resouce_code(
    filename: str,
    file_version: str,
    file_desc: str,
    icon_path: str,
    product_name: str,
    product_version: str,
    company_name: str,
) -> str:
    template = """\
1 ICON "%(icon_path)s"
1 VERSIONINFO
FILEVERSION     %(file_version_list)s
PRODUCTVERSION  %(product_version_list)s
FILEOS 0x4
FILETYPE 0x1
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904E4"
        BEGIN
            VALUE "CompanyName",      "%(company_name)s"
            VALUE "FileDescription",  "%(file_desc)s"
            VALUE "FileVersion",      "%(file_version)s"
            VALUE "InternalName",     "%(internal_name)s"
            VALUE "OriginalFilename", "%(filename)s"
            VALUE "ProductName",      "%(product_name)s"
            VALUE "ProductVersion",   "%(product_version)s"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1252
    END
END
"""

    def to_ver_list(v: str) -> str:
        v = v.split("-", maxsplit=1)[0]
        return ",".join(map(str, (list(map(int, v.split("."))) + [0] * 4)[:4]))

    file_version_list = to_ver_list(file_version)
    product_version_list = to_ver_list(product_version)

    return template % {
        "icon_path": icon_path,
        "file_version_list": file_version_list,
        "product_version_list": product_version_list,
        "file_version": file_version,
        "product_version": product_version,
        "company_name": company_name,
        "filename": filename,
        "internal_name": os.path.splitext(filename)[0],
        "product_name": product_name,
        "file_desc": file_desc,
    }


def build_launcher(
    out_path: str,
    icon_path: str,
    file_desc: str,
    product_name: str,
    product_version: str,
    company_name: str,
    *,
    is_gui: bool,
    debug: bool,
) -> None:
    src_ico = os.path.abspath(icon_path)
    target = os.path.abspath(out_path)

    file_version = product_version

    dir_ = os.getcwd()
    temp = tempfile.mkdtemp()
    try:
        os.chdir(temp)
        with open("launcher.c", "w") as h:
            h.write(get_launcher_code(debug=debug))
        shutil.copyfile(src_ico, "launcher.ico")
        with open("launcher.rc", "w") as h:
            h.write(
                get_resouce_code(
                    os.path.basename(target),
                    file_version,
                    file_desc,
                    "launcher.ico",
                    product_name,
                    product_version,
                    company_name,
                )
            )

        build_resource("launcher.rc", "launcher.res")
        build_exe("launcher.c", "launcher.res", target, is_gui=is_gui)
    finally:
        os.chdir(dir_)
        shutil.rmtree(temp)


def main() -> None:
    argv = sys.argv

    version = argv[1]
    target = argv[2]

    company_name = "Gajim"
    misc = os.path.dirname(os.path.realpath(__file__))

    build_launcher(
        os.path.join(target, "Gajim.exe"),
        os.path.join(misc, "gajim.ico"),
        "Gajim",
        "Gajim",
        version,
        company_name,
        is_gui=True,
        debug=False,
    )

    build_launcher(
        os.path.join(target, "Gajim-Debug.exe"),
        os.path.join(misc, "gajim.ico"),
        "Gajim",
        "Gajim",
        version,
        company_name,
        is_gui=False,
        debug=True,
    )

    # build_launcher(
    #     os.path.join(target, "history_manager.exe"),
    #     os.path.join(misc, "gajim.ico"), "History Manager", "History Manager",
    #     version, company_name, 'history_manager.py', True)


if __name__ == "__main__":
    main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import unittest
from datetime import datetime
from datetime import UTC

from nbxmpp.protocol import JID

from gajim.common import app
from gajim.common.settings import Settings
from gajim.common.storage.archive.const import ChatDirection
from gajim.common.storage.archive.const import MessageState
from gajim.common.storage.archive.const import MessageType
from gajim.common.storage.archive.models import Message
from gajim.common.storage.archive.storage import MessageArchiveStorage


class MessagesTest(unittest.TestCase):
    def setUp(self) -> None:
        self._archive = MessageArchiveStorage(in_memory=True)
        self._archive.init()

        self._account = "testacc1"
        self._account_jid = JID.from_string("user@domain.org")
        self._remote_jid = JID.from_string("remote@jid.org")
        self._occupant_id = "occupantid1"
        self._init_settings()

    def tearDown(self) -> None:
        self._archive.shutdown()

    def _init_settings(self) -> None:
        app.settings = Settings(in_memory=True)
        app.settings.init()
        app.settings.add_account("testacc1")
        app.settings.set_account_setting("testacc1", "address", "user@domain.org")

    def _create_base_message(
        self,
        message_id: str,
        stanza_id: str | None,
        direction: ChatDirection = ChatDirection.INCOMING,
    ) -> Message:
        return Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            resource="someres1",
            type=MessageType.CHAT,
            direction=direction,
            timestamp=datetime.now(UTC),
            state=MessageState.ACKNOWLEDGED,
            id=message_id,
            stanza_id=stanza_id,
            text="message",
        )


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations

from typing import Any

from . import constants as constants

class QRCode:
    def __init__(
        self,
        version: int | None,
        error_correction: int,
        box_size: int,
        border: int,
        image_factory: Any = None,
        mask_pattern: Any = None,
    ) -> None: ...
    def add_data(self, data: str) -> None: ...
    def make(self, fit: bool) -> None: ...
    def make_image(self, image_factory: Any = None, **kwargs: Any) -> Any: ...



from __future__ import annotations

from typing import Any

class NSLocale:
    @classmethod
    def currentLocale(cls) -> Any: ...

class NSSound:
    @classmethod
    def alloc(cls) -> NSSound: ...
    def initWithContentsOfFile_byReference_(self, url: str, byref: bool) -> None: ...
    def play(self) -> None: ...

class NSWorkspace:
    @classmethod
    def sharedWorkspace(cls) -> Any: ...
    def activateFileViewerSelectingURLs_(self, urls: list[Any]) -> None: ...



"""
This type stub file was generated by pyright.
"""

import typing

from . import _base

class Icon(_base.Icon):
    _HWND_TO_ICON: typing.Any = ...
    def __init__(self, *args: typing.Any, **kwargs: typing.Any) -> None: ...
    def __del__(self) -> None: ...



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import unittest
from datetime import datetime
from datetime import UTC

from nbxmpp.protocol import JID
from sqlalchemy import select

from gajim.common import app
from gajim.common.helpers import get_uuid
from gajim.common.settings import Settings
from gajim.common.storage.archive.const import ChatDirection
from gajim.common.storage.archive.const import MessageState
from gajim.common.storage.archive.const import MessageType
from gajim.common.storage.archive.models import Message
from gajim.common.storage.archive.models import MessageError
from gajim.common.storage.archive.storage import MessageArchiveStorage
from gajim.common.util.datetime import utc_now


class ErrorTest(unittest.TestCase):
    def setUp(self) -> None:
        self._archive = MessageArchiveStorage(in_memory=True)
        self._archive.init()

        self._account = "testacc1"
        self._account_jid = JID.from_string("user@domain.org")
        self._remote_jid = JID.from_string("remote@jid.org")
        self._occupant_id = "occupantid1"
        self._init_settings()

    def tearDown(self) -> None:
        self._archive.shutdown()

    def _init_settings(self) -> None:
        app.settings = Settings(in_memory=True)
        app.settings.init()
        app.settings.add_account("testacc1")
        app.settings.set_account_setting("testacc1", "address", "user@domain.org")

    def test_insert_error(self) -> None:
        error_data1 = MessageError(
            account_=self._account,
            remote_jid_=self._remote_jid,
            message_id="1",
            by=JID.from_string("some@domain.com"),
            type="modify",
            text="Text 1",
            condition="not-acceptable",
            condition_text="Condition Data 1",
            timestamp=utc_now(),
        )

        pk = self._archive.insert_row(error_data1)

        with self._archive.get_session() as s:
            error = s.scalar(select(MessageError).where(MessageError.pk == pk))
        assert error is not None

        self.assertEqual(error.by, "some@domain.com")
        self.assertIsInstance(error.by, JID)
        self.assertEqual(error.text, "Text 1")
        self.assertEqual(error.condition, "not-acceptable")
        self.assertEqual(error.condition_text, "Condition Data 1")
        self.assertEqual(error.type, "modify")

        error_data2 = MessageError(
            account_=self._account,
            remote_jid_=self._remote_jid,
            message_id="1",
            by=JID.from_string("some@domain.com"),
            type="cancel",
            text="Text 2",
            condition="not-acceptable",
            condition_text="Condition Data 2",
            timestamp=utc_now(),
        )

        pk = self._archive.insert_row(error_data2, ignore_on_conflict=True)
        self.assertEqual(pk, -1)

    def test_error_join(self) -> None:
        message_data = Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            resource="someres1",
            type=MessageType.CHAT,
            direction=ChatDirection.INCOMING,
            timestamp=datetime.now(UTC),
            state=MessageState.ACKNOWLEDGED,
            id="1",
            stanza_id=get_uuid(),
            text="message",
        )

        pk = self._archive.insert_object(message_data)

        error_data1 = MessageError(
            account_=self._account,
            remote_jid_=self._remote_jid,
            message_id="1",
            by=JID.from_string("some@domain.com"),
            type="modify",
            text="Text 1",
            condition="not-acceptable",
            condition_text="Condition Data 1",
            timestamp=utc_now(),
        )
        self._archive.insert_row(error_data1)

        message = self._archive.get_message_with_pk(pk)

        assert message is not None
        assert message.error is not None
        self.assertEqual(message.error.by, "some@domain.com")
        self.assertIsInstance(message.error.by, JID)
        self.assertEqual(message.error.text, "Text 1")
        self.assertEqual(message.error.condition, "not-acceptable")
        self.assertEqual(message.error.condition_text, "Condition Data 1")
        self.assertEqual(message.error.type, "modify")


if __name__ == "__main__":
    unittest.main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from functools import partial
from unittest.mock import MagicMock

from gi.repository import Gtk
from nbxmpp.client import Client as NBXMPPClient

from gajim.common import app

from gajim.gtk.account_wizard import AccountWizard
from gajim.gtk.account_wizard import WizardSuccessPage

from . import util

ACCOUNT = "testacc1"


def _on_login_successful(
    self: AccountWizard, client: NBXMPPClient, _signal_name: str
) -> None:
    account = self._generate_account_name(client.domain)  # type: ignore
    app.settings.add_account(account)
    self.get_page("success").set_account(account)
    self.show_page("success", Gtk.StackTransitionType.SLIDE_LEFT)
    self._disconnect()  # type: ignore


def _save_config(self: WizardSuccessPage) -> None:
    assert self._account is not None  # type: ignore
    app.settings.set_account_setting(self._account, "account_color", self._color)  # type: ignore
    if self._label:  # type: ignore
        app.settings.set_account_setting(self._account, "account_label", self._label)  # type: ignore
    app.css_config.refresh()


util.init_settings()

app.get_client = MagicMock()

app.get_jid_from_account = MagicMock(return_value="testjid")

app.cert_store = MagicMock()

app.css_config = MagicMock()
app.css_config.prefer_dark = MagicMock(return_value=False)

window = AccountWizard()
window._on_login_successful = partial(_on_login_successful, window)  # type: ignore

success_page = window.get_page("success")
success_page._save_config = partial(_save_config, success_page)  # type: ignore
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-only

from typing import Any

from unittest.mock import MagicMock

import gi

from gajim.common.file_transfer_manager import FileTransferManager
from gajim.common.util.preview import GeoPreview
from gajim.common.util.preview import get_preview_data
from gajim.common.util.preview import UrlPreview

gi.require_version("Gst", "1.0")
gi.require_version("GstPbutils", "1.0")

from gi.repository import Adw
from gi.repository import Gst
from gi.repository import Gtk

from gajim.common import app
from gajim.common import configpaths
from gajim.common import logging_helpers
from gajim.common.const import CSSPriority

from gajim.gtk.dropdown import GajimDropDown
from gajim.gtk.dropdown import KeyValueItem
from gajim.gtk.preview.geo import GeoPreviewWidget
from gajim.gtk.preview.preview import PreviewWidget
from gajim.gtk.window import GajimAppWindow

from . import util

util.load_style("gajim.css", CSSPriority.APPLICATION)

ACCOUNT = "me@example.org"

PREVIEW_TYPES = {
    "geo: URI": "geo:50.3333,24.5555",
    "Image URL": "https://gajim.org/img/screenshots/server-info.png",
    "Animated Gif URL": "https://gajim.org/img/gajim-test.gif",
    "Animated WebP URL": "https://gajim.org/img/gajim-test.webp",
    "Animated Avif URL": "https://gajim.org/img/gajim-test.avif",
    "Animated PNG URL": "https://gajim.org/img/gajim-test.png",
    "Audio URL": "https://dev.gajim.org/gajim/gajim/-/wikis/uploads/dec966d89848453df07e0bd9b2ebc3d3/Gajim.ogg",
    "PDF URL": "https://www.rfc-editor.org/rfc/pdfrfc/rfc6120.txt.pdf",
}


class TestPreview(GajimAppWindow):
    def __init__(self) -> None:
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=600,
            default_height=600,
            add_window_padding=True,
            header_bar=True,
        )

        self._box = Gtk.Box(
            halign=Gtk.Align.CENTER,
            valign=Gtk.Align.CENTER,
            orientation=Gtk.Orientation.VERTICAL,
            hexpand=True,
            spacing=12,
        )
        self.set_child(self._box)

        self._preview_widget = None

        drop_down: GajimDropDown[str] = GajimDropDown(list(PREVIEW_TYPES.keys()))
        drop_down.connect("notify::selected", self._on_preview_type_selected)
        drop_down.select_key(list(PREVIEW_TYPES.keys())[1])
        self._box.append(drop_down)

    def _on_preview_type_selected(
        self, drop_down: GajimDropDown[str], *args: Any
    ) -> None:
        selected_type = drop_down.get_selected_item()
        assert isinstance(selected_type, KeyValueItem)
        uri_data = PREVIEW_TYPES[selected_type.key]

        is_outgoing = True
        muc_context = None

        if self._preview_widget is not None:
            self._box.remove(self._preview_widget)

        match preview := get_preview_data(uri_data, []):
            case GeoPreview():
                print("geo")
                self._preview_widget = GeoPreviewWidget(preview)
            case UrlPreview():
                print("url")
                self._preview_widget = PreviewWidget(
                    ACCOUNT, preview, is_outgoing, muc_context
                )
            case _:
                print(None)
                self._preview_widget = None

        if self._preview_widget is not None:
            self._box.prepend(self._preview_widget)

    def _cleanup(self) -> None:
        pass


Gst.init()
Adw.init()

app.init_process_pool()
app.window = Gtk.Window()
app.is_installed = MagicMock(return_value=True)

logging_helpers.set_loglevels("gajim=DEBUG")

util.init_settings()
app.settings.add_account(ACCOUNT)
app.settings.set("preview_size", 400)
app.settings.set("preview_allow_all_images", True)

configpaths.set_config_root(str(configpaths.get_temp_dir()))
configpaths.init()
configpaths.create_paths()

app.ftm = FileTransferManager()

window = TestPreview()
window.show()

util.run_app()



#!/usr/bin/env python3
import os
import shutil
from argparse import ArgumentParser
from subprocess import run

ICON_SVG = "gajim/data/icons/hicolor/scalable/apps/gajim.svg"


def create_icns(icon_path: str) -> None:
    tmpdir = "Gajim.iconset"
    if os.path.isdir(tmpdir):
        shutil.rmtree(tmpdir)
    os.mkdir(tmpdir)

    for size_pt in [16, 32, 128, 256, 512]:
        for scale in [1, 2]:
            size_px = scale * size_pt
            scale_txt = f"@{scale}" if scale != 1 else ""
            png_fn = f"icon_{size_pt}x{size_pt}{scale_txt}.png"
            png_path = os.path.join(tmpdir, png_fn)
            run(
                [
                    "inkscape",
                    "-z",
                    "-e",
                    png_path,
                    "-w",
                    str(size_px),
                    "-h",
                    str(size_px),
                    "-y",
                    "0",
                    ICON_SVG,
                ],
                check=True,
            )
    run(["iconutil", "-c", "icns", "-o", icon_path, tmpdir], check=True)

    shutil.rmtree(tmpdir)


if __name__ == "__main__":
    parser = ArgumentParser(
        description="Create a macOS .icns icon. Requires Inkscape and iconutil (macOS)."
    )
    parser.add_argument("output", help="bundle output location")
    args = parser.parse_args()

    create_icns(args.output)



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from pathlib import Path

from gi.repository import Gtk

from gajim.gtk.filechoosers import FileChooserButton
from gajim.gtk.filechoosers import Filter
from gajim.gtk.window import GajimAppWindow

from . import util


class TestFileChooserButton(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=600,
            default_height=600,
            add_window_padding=True,
            header_bar=True,
        )

        box = Gtk.Box(halign=Gtk.Align.CENTER, valign=Gtk.Align.CENTER, hexpand=True)
        self.set_child(box)

        file_chooser_button = FileChooserButton(
            filters=[
                Filter(name="All files", patterns=["*"]),
                Filter(name="WAV Sounds", suffixes=["wav"]),
            ]
        )
        file_chooser_button.connect("path-picked", self._on_file_picked)
        box.append(file_chooser_button)

    def _on_file_picked(
        self, _file_chooser_button: FileChooserButton, file_paths: list[Path]
    ) -> None:
        print(file_paths)

    def _cleanup(self) -> None:
        pass


util.init_settings()

window = TestFileChooserButton()
window.show()

util.run_app()



from __future__ import annotations

import typing

@typing.final
class AppInfo_Static(type):
    @property
    def current(cls) -> typing.Optional[AppInfo]: ...

@typing.final
class AppInfo(metaclass=AppInfo_Static):
    @property
    def app_user_model_id(self) -> str: ...



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import Any

import datetime as dt
import random
import string
from collections.abc import Iterator
from unittest.mock import MagicMock

from gi.repository import Gdk
from gi.repository import GLib
from gi.repository import Gtk
from nbxmpp.const import PresenceShow
from nbxmpp.protocol import JID
from nbxmpp.structs import BookmarkData
from nbxmpp.structs import RosterItem

from gajim.common import app
from gajim.common.const import AvatarSize
from gajim.common.const import CSSPriority
from gajim.common.modules.contacts import BareContact
from gajim.common.modules.contacts import GroupchatContact

from gajim.gtk.avatar import convert_surface_to_texture
from gajim.gtk.avatar import generate_default_avatar
from gajim.gtk.start_chat import StartChatDialog

from . import util

ACCOUNTS_COUNT = 2
CONTACTS_PER_ACCOUNT_COUNT = 10
CONTACTS_COUNT = int(CONTACTS_PER_ACCOUNT_COUNT / 2)
GROUPCHATS_COUNT = int(CONTACTS_PER_ACCOUNT_COUNT / 2)
CONTACT_GROUPS = {"Group A", "Group B", "Group C with long name"}
STATUS_MESSAGES = [
    "Status Message very very very very very long\nwith newline",
    "Status message",
    "",
]


class TestRosterModule:
    def __init__(self, account: str) -> None:
        self._account = account
        self._roster: dict[JID, RosterItem] = {}
        for i in range(CONTACTS_COUNT):
            jid = JID.from_string(f"test{i}@{self._account}.org")
            self._roster[jid] = RosterItem(
                jid=jid,
                name=f"name{i}",
                ask=None,
                subscription=None,
                approved=None,
                groups=set(),
            )

    def get_groups(self) -> set[str]:
        return CONTACT_GROUPS

    def iter(self) -> Iterator[tuple[JID, RosterItem]]:
        yield from self._roster.items()


class TestBookmarksModule:
    def __init__(self, account: str) -> None:
        self._account = account
        self._bookmarks: dict[JID, BookmarkData] = {}

        for i in range(GROUPCHATS_COUNT):
            jid = JID.from_string(f"groupchat{i}@conference.{self._account}.org")
            self._bookmarks[jid] = BookmarkData(
                jid=jid,
                name=f"name{i}",
            )

    @property
    def bookmarks(self) -> list[BookmarkData]:
        return list(self._bookmarks.values())


class TestContactsModule:
    def __init__(self, account: str) -> None:
        self._account = account

        self._avatars: list[Gdk.Texture] = []
        alphabet = list(string.ascii_uppercase)
        for _i in range(5):
            color = (
                random.randrange(100) / 100,
                random.randrange(100) / 100,
                random.randrange(100) / 100,
            )
            avatar = generate_default_avatar(
                random.choice(alphabet), color, AvatarSize.CALL_BIG, 1
            )
            self._avatars.append(convert_surface_to_texture(avatar))

        self._contact_index = 0
        self._groupchat_contact_index = 0

    def get_contact(self, jid: str | JID, groupchat: bool = False) -> BareContact:
        if groupchat:
            spec_set = GroupchatContact
            contact_name = f"Group Chat {self._groupchat_contact_index}"
            self._groupchat_contact_index += 1
        else:
            contact_name = f"Contact {self._contact_index}"
            spec_set = BareContact
            self._contact_index += 1

        contact = MagicMock(spec_set=spec_set)
        contact.jid = jid
        contact.name = contact_name
        contact.get_avatar = MagicMock(return_value=random.choice(self._avatars))

        contact.account = self._account
        contact.is_groupchat = groupchat

        if not groupchat:
            contact.show = PresenceShow.ONLINE
            contact.status = random.choice(STATUS_MESSAGES)
            contact.idle_datetime = dt.datetime.now()
            contact.groups = set(random.sample(sorted(CONTACT_GROUPS), 1))

        return contact


class TestClient:
    def __init__(self, account: str) -> None:
        self._account = account

        self._contacts_module = TestContactsModule(self._account)
        self._bookmarks_module = TestBookmarksModule(self._account)
        self._roster_module = TestRosterModule(self._account)

    def get_own_jid(self) -> JID:
        return JID.from_string(self._account)

    def get_module(self, module: str) -> Any:
        if module == "Contacts":
            return self._contacts_module
        if module == "Bookmarks":
            return self._bookmarks_module
        if module == "Roster":
            return self._roster_module
        return MagicMock()


def _get_dynamic_class(account: str) -> str:
    color = app.settings.get_account_setting(account, "account_color")
    css_class = f"gajim_class_{account}"
    css = f".{css_class} {{ background-color: {color}; }}\n"
    dynamic_provider = Gtk.CssProvider()
    dynamic_provider.load_from_bytes(GLib.Bytes.new(css.encode("utf-8")))
    display = Gdk.Display.get_default()
    assert display is not None
    Gtk.StyleContext.add_provider_for_display(
        display, dynamic_provider, CSSPriority.APPLICATION
    )
    return css_class


accounts: list[list[str]] = []
for i in range(ACCOUNTS_COUNT):
    accounts.append(
        [f"account{i}", f"Account {i}"],
    )

util.init_settings()

for account in accounts:
    app.settings.add_account(account[0])
    app.settings.set_account_setting(
        account[0],
        "account_color",
        f"rgb({random.randrange(255)}, {random.randrange(255)}, {random.randrange(255)})",  # noqa: E501
    )
    app.settings.set_account_setting(account[0], "account_label", account[1])


app.css_config = MagicMock()
app.css_config.get_dynamic_class = MagicMock(side_effect=_get_dynamic_class)

app.get_client = MagicMock(side_effect=TestClient)

app.get_enabled_accounts_with_labels = MagicMock(return_value=accounts)


window = StartChatDialog()
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import unittest
from datetime import datetime
from datetime import UTC

from nbxmpp.protocol import JID
from sqlalchemy import select

from gajim.common import app
from gajim.common.helpers import get_uuid
from gajim.common.settings import Settings
from gajim.common.storage.archive.const import ChatDirection
from gajim.common.storage.archive.const import MessageState
from gajim.common.storage.archive.const import MessageType
from gajim.common.storage.archive.models import Encryption
from gajim.common.storage.archive.models import Message
from gajim.common.storage.archive.models import MessageError
from gajim.common.storage.archive.models import OOB
from gajim.common.storage.archive.models import Reply
from gajim.common.storage.archive.models import SecurityLabel
from gajim.common.storage.archive.storage import MessageArchiveStorage
from gajim.common.util.datetime import utc_now


class ForeignKeyTest(unittest.TestCase):
    def setUp(self) -> None:
        self._archive = MessageArchiveStorage(in_memory=True)
        self._archive.init()

        self._account = "testacc1"
        self._account_jid = JID.from_string("user@domain.org")
        self._remote_jid = JID.from_string("remote@jid.org")
        self._occupant_id = "occupantid1"
        self._init_settings()

    def tearDown(self) -> None:
        self._archive.shutdown()

    def _init_settings(self) -> None:
        app.settings = Settings(in_memory=True)
        app.settings.init()
        app.settings.add_account("testacc1")
        app.settings.set_account_setting("testacc1", "address", "user@domain.org")

    def test_message_delete_cascade(self) -> None:
        oob_data1 = OOB(url="https://www.test.com", description="somedesc")
        oob_data2 = OOB(url="https://www.othertest.com", description="otherdesc")

        reply_data = Reply(
            id="123",
            to=JID.from_string("somejid@jid.com"),
        )

        enc_data = Encryption(protocol="OMEMO", key="testkey", trust=2)

        sec_data = SecurityLabel(
            account_=self._account,
            remote_jid_=self._remote_jid,
            updated_at=datetime.fromtimestamp(0, UTC),
            label_hash="somehash",
            displaymarking="SECRET",
            fgcolor="black",
            bgcolor="red",
        )

        message_data = Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            resource="someres1",
            type=MessageType.CHAT,
            direction=ChatDirection.INCOMING,
            timestamp=datetime.now(UTC),
            state=MessageState.ACKNOWLEDGED,
            id="1",
            stanza_id=get_uuid(),
            text="message",
            encryption_=enc_data,
            security_label_=sec_data,
            oob=[oob_data1, oob_data2],
            reply=reply_data,
        )

        pk = self._archive.insert_object(message_data)

        error_data = MessageError(
            account_=self._account,
            remote_jid_=self._remote_jid,
            message_id="1",
            by=JID.from_string("some@jid.com"),
            type="modify",
            text="Some error text",
            condition="not-acceptable",
            condition_text="Some Application Text",
            timestamp=utc_now(),
        )

        self._archive.insert_row(error_data)

        message = self._archive.get_message_with_pk(pk)

        assert message is not None
        assert message.oob is not None
        self.assertEqual(message.oob[0].description, "somedesc")
        self.assertEqual(message.oob[1].description, "otherdesc")

        assert message.reply is not None
        self.assertEqual(message.reply.id, "123")
        self.assertEqual(message.reply.to, JID.from_string("somejid@jid.com"))
        self.assertIsInstance(message.reply.to, JID)

        self.assertEqual(message.account.jid, self._account_jid)
        self.assertEqual(message.remote.jid, self._remote_jid)

        assert message.encryption is not None
        self.assertEqual(message.encryption.key, "testkey")

        assert message.security_label is not None
        self.assertEqual(message.security_label.displaymarking, "SECRET")

        assert message.error is not None
        self.assertEqual(message.error.by, "some@jid.com")
        self.assertIsInstance(message.error.by, JID)
        self.assertEqual(message.error.text, "Some error text")
        self.assertEqual(message.error.condition, "not-acceptable")
        self.assertEqual(message.error.condition_text, "Some Application Text")
        self.assertEqual(message.error.type, "modify")

        self._archive.delete_message(message.pk)

        stmts = [
            select(Reply),
            select(OOB),
            select(Message),
        ]

        for stmt in stmts:
            with self._archive.get_session() as s:
                res = s.execute(stmt).one_or_none()
            assert res is None


if __name__ == "__main__":
    unittest.main()



"""
This type stub file was generated by pyright.
"""

from typing import Optional
from typing import TypeVar

from datetime import datetime

from winrt.windows.ui.notifications import ScheduledToastNotification
from winrt.windows.ui.notifications import ToastNotification
from winrt.windows.ui.notifications import ToastNotifier

from .toast import Toast

ToastNotificationT = TypeVar(
    "ToastNotificationT", ToastNotification, ScheduledToastNotification
)

class BaseWindowsToaster:
    """
    Wrapper to simplify WinRT's ToastNotificationManager

    :param applicationText: Text to display the application as
    """

    applicationText: str
    notifierAUMID: Optional[str]
    toastNotifier: ToastNotifier
    def __init__(self, applicationText: str) -> None: ...
    def show_toast(self, toast: Toast) -> None:
        """
        Displays the specified toast notification.
        If `toast` has already been shown, it will pop up again, but make no new sections in the action center

        :param toast: Toast to display
        """
        ...

    def update_toast(self, toast: Toast) -> bool:
        """
        Update the passed notification data with the new data in the clas

        :param toast: Toast to update
        :type toast: Toast
        :return: Whether the update succeeded
        """
        ...

    def schedule_toast(self, toast: Toast, displayTime: datetime) -> None:
        """
        Schedule the passed notification toast. Warning: scheduled toasts cannot be updated or activated (i.e. on_X)

        :param toast: Toast to display
        :type toast: Toast
        :param displayTime: Time to display the toast on
        :type displayTime: datetime
        """
        ...

    def unschedule_toast(self, toast: Toast) -> None:
        """
        Unschedule the passed notification toast

        :raises: ToastNotFoundError: If the toast could not be found
        """
        ...

    def clear_toasts(self) -> None:
        """
        Clear toasts popped by this toaster
        """
        ...

    def clear_scheduled_toasts(self) -> None:
        """
        Clear all scheduled toasts set for the toaster
        """
        ...

class WindowsToaster(BaseWindowsToaster):
    """
    Basic toaster, used to display toasts without actions and/or inputs.
    If you need to use them, see :class:`InteractableWindowsToaster`

    :param applicationText: Text to display the application as
    """

    __InteractableWarningMessage = ...
    def __init__(self, applicationText: str) -> None: ...
    def show_toast(self, toast: Toast) -> None: ...

class InteractableWindowsToaster(BaseWindowsToaster):
    """
    :class:`WindowsToaster`, but uses a AUMID to support actions. Actions require a recognised AUMID to trigger
    on_activated, otherwise it triggers on_dismissed with no arguments

    :param applicationText: Text to display the application as
    :param notifierAUMID: AUMID to use. Defaults to Command Prompt. To use a custom AUMID, see one of the scripts
    """

    def __init__(
        self, applicationText: str, notifierAUMID: Optional[str] = ...
    ) -> None: ...



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import unittest
from datetime import datetime
from datetime import UTC

from nbxmpp.protocol import JID
from sqlalchemy import select

from gajim.common import app
from gajim.common.helpers import get_uuid
from gajim.common.settings import Settings
from gajim.common.storage.archive.const import ChatDirection
from gajim.common.storage.archive.const import MessageState
from gajim.common.storage.archive.const import MessageType
from gajim.common.storage.archive.models import Message
from gajim.common.storage.archive.models import SecurityLabel
from gajim.common.storage.archive.storage import MessageArchiveStorage


class SecurityLabelsTest(unittest.TestCase):
    def setUp(self) -> None:
        self._archive = MessageArchiveStorage(in_memory=True)
        self._archive.init()

        self._account_jid = JID.from_string("user@domain.org")
        self._account = "testacc1"
        self._remote_jid = JID.from_string("remote@jid.org")
        self._init_settings()

    def tearDown(self) -> None:
        self._archive.shutdown()

    def _init_settings(self) -> None:
        app.settings = Settings(in_memory=True)
        app.settings.init()
        app.settings.add_account("testacc1")
        app.settings.set_account_setting("testacc1", "address", "user@domain.org")

    def test_security_labels_join(self):
        sec_data = SecurityLabel(
            account_=self._account,
            remote_jid_=self._remote_jid,
            updated_at=datetime.fromtimestamp(0, UTC),
            label_hash="label1hash",
            displaymarking="SECRET",
            fgcolor="black",
            bgcolor="red",
        )

        message_data = Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            type=MessageType.CHAT,
            direction=ChatDirection.INCOMING,
            timestamp=datetime.fromtimestamp(0, UTC),
            state=MessageState.ACKNOWLEDGED,
            resource="res",
            text="Some Message",
            id="messageid1",
            stanza_id=get_uuid(),
            security_label_=sec_data,
        )

        message_pk = self._archive.insert_object(message_data)

        message = self._archive.get_message_with_pk(message_pk)

        assert message is not None
        assert message.security_label is not None

        self.assertEqual(message.security_label.displaymarking, "SECRET")
        self.assertEqual(message.security_label.fgcolor, "black")
        self.assertEqual(message.security_label.bgcolor, "red")

    def test_security_labels_update(self):
        sec_data1 = SecurityLabel(
            account_=self._account,
            remote_jid_=self._remote_jid,
            updated_at=datetime.fromtimestamp(0, UTC),
            label_hash="label1hash",
            displaymarking="SECRET",
            fgcolor="black",
            bgcolor="red",
        )

        sec_data2 = SecurityLabel(
            account_=self._account,
            remote_jid_=self._remote_jid,
            updated_at=datetime.fromtimestamp(1, UTC),
            label_hash="label1hash",
            displaymarking="NOT SECRET",
            fgcolor="white",
            bgcolor="blue",
        )

        pk1 = self._archive.upsert_row(sec_data1)
        pk2 = self._archive.upsert_row(sec_data2)
        pk3 = self._archive.upsert_row(sec_data1)

        self.assertEqual(pk1, pk2)
        self.assertEqual(pk2, pk3)

        with self._archive.get_session() as s:
            res = s.scalar(select(SecurityLabel).where(SecurityLabel.pk == pk3))
        assert res is not None

        self.assertEqual(res.displaymarking, "NOT SECRET")
        self.assertEqual(res.fgcolor, "white")
        self.assertEqual(res.bgcolor, "blue")
        self.assertEqual(res.updated_at, datetime.fromtimestamp(1, UTC))


if __name__ == "__main__":
    unittest.main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from unittest.mock import MagicMock

from gi.repository import Gtk
from nbxmpp.modules.discovery import parse_disco_info
from nbxmpp.protocol import Iq
from nbxmpp.structs import MucSubject

from gajim.common import app

from gajim.gtk.groupchat_info import GroupChatInfoScrolled
from gajim.gtk.window import GajimAppWindow

from . import util

stanza = Iq(
    node="""
<iq xmlns="jabber:client" xml:lang="de-DE" to="user@user.us" from="asd@conference.temptatio.dev" type="result" id="67284933-e526-41f3-8309-9d9475cf9c74">
    <query
        xmlns="http://jabber.org/protocol/disco#info">
        <identity name="ipsum dolor sit amet, consetetur sadipscing elitr sed diam nonumy eirmod tempor invidunt" type="text" category="conference" />
        <feature var="vcard-temp" />
        <feature var="http://jabber.org/protocol/muc" />
        <feature var="http://jabber.org/protocol/disco#info" />
        <feature var="http://jabber.org/protocol/disco#items" />
        <feature var="muc_temporary" />
        <feature var="muc_moderated" />
        <feature var="muc_open" />
        <feature var="muc_hidden" />
        <feature var="muc_nonanonymous" />
        <feature var="muc_passwordprotected" />
        <feature var="urn:xmpp:mam:2" />
        <feature var="muc_public" />
        <feature var="muc_persistent" />
        <feature var="muc_membersonly" />
        <feature var="muc_semianonymous" />
        <feature var="muc_unmoderated" />
        <feature var="muc_unsecured" />
        <x type="result"
            xmlns="jabber:x:data">
            <field var="FORM_TYPE" type="hidden">
                <value>http://jabber.org/protocol/muc#roominfo</value>
            </field>
            <field var="muc#roominfo_occupants" type="text-single" label="Number of occupants">
                <value>1</value>
            </field>
            <field var="muc#roomconfig_roomname" type="text-single" label="Natural-Language Room Name">
                <value>ipsum dolor sit amet, consetetur sadipscing elitr sed diam nonumy eirmod tempor invidunt</value>
            </field>
            <field var="muc#roominfo_description" type="text-single" label="Raum Beschreibung">
                <value>Lorem ipsum dolor sit amet, consetetur sadipscing elitr sed diam nonumy eirmod tempor invidunt ut labore et dolore magna</value>
            </field>
            <field var="muc#roominfo_contactjid" type="jid-multi" label="Contact Addresses (normally, room owner or owners)">
                <value>userA@user.us</value>
                <value>userB@user.us</value>
            </field>
            <field var="muc#roominfo_changesubject" type="boolean" label="Occupants May Change the Subject">
                <value>1</value>
            </field>
            <field var="muc#roomconfig_allowinvites" type="boolean" label="Occupants are allowed to invite others">
                <value>1</value>
            </field>
            <field var="muc#roomconfig_allowpm" type="list-single" label="Roles that May Send Private Messages">
                <value>anyone</value>
                <option label="Anyone">
                    <value>anyone</value>
                </option>
                <option label="Anyone with Voice">
                    <value>participants</value>
                </option>
                <option label="Moderators Only">
                    <value>moderators</value>
                </option>
                <option label="Nobody">
                    <value>none</value>
                </option>
            </field>
            <field var="muc#roominfo_lang" type="text-single" label="Natural Language for Room Discussions">
                <value>de</value>
            </field>
            <field type="text-single" var="muc#roominfo_logs">
                <value>https://logs.xmpp.org/xsf/</value>
            </field>
        </x>
    </query>
</iq>"""  # type: ignore  # noqa: E501
)


subject = (
    "Lorem ipsum dolor sit amet, consetetur sadipscing elitr sed "
    "diam nonumy eirmod tempor invidunt ut labore et dolore magna"
)

disco_info = parse_disco_info(stanza)


class TestGroupchatInfo(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=700,
            default_height=700,
            add_window_padding=True,
            header_bar=True,
        )

        self._main_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=18)
        self._main_box.set_valign(Gtk.Align.FILL)

        self._muc_info_box = GroupChatInfoScrolled()
        self._muc_info_box.set_vexpand(True)

        self._main_box.append(self._muc_info_box)

        self.set_child(self._main_box)
        self._muc_info_box.set_from_disco_info(disco_info)
        self._muc_info_box.set_subject(
            MucSubject(text=subject, author="someone", timestamp=None)
        )

    def _cleanup(self) -> None:
        pass


app.css_config = MagicMock()
app.css_config.get_value = MagicMock(return_value="rgb(100, 100, 255)")

util.init_settings()

window = TestGroupchatInfo()
window.show()

util.run_app()
